pro auto_plot_chi2_binary_radec_scale,ra_array,dec_array,chi2_array,dof,rainbow=rainbow,markpos=markpos,ptitle=ptitle,xrange=xrange,yrange=yrange,plot_cont=plot_cont

; Plot chi2 surface resulting for binary grid search routine. This program 
; scales the chi2 array so that reduced chi2 = 1 for the best fit solution.
;
; Call to program:
; IDL> auto_plot_chi2_binary_radec_scale, ra_array, dec_array, chi2_array, dof, $
;         /rainbow,xrange=[min_sepRA,max_sepRA],yrange=[min_sepDEC,max_sepDEC]
;
; Input Parameters:
;   ra_array:   Two dimensional array containing sepRA values for each
;               step in the grid (num_sepRA,num_sepDEC).
;   dec_array:  Two dimensional array containing sepDEC values for each
;               step in the grid (num_sepRA,num_sepDEC).
;   chi2_array: Two dimensional array containing chi2 values for each
;               step in the grid (num_sepRA,num_sepDEC).
;   DOF:        Number of degrees of freedom = NUM_V2 + NUM_T3 - N_PARAM
;
; Optional Input Paramaeters:
;   rainbow:    Use 7 colors to plot chi2 surfaces rather than 3.
;   markpos:    [sepRA,sepDEC] Two dimensional array to mark location on map.
;   xrange:     Plotting range in x [min_sepRA,max_sepRA].
;   yrange:     Plotting range in y [min_sepRA,max_sepRA].
;   ptitle:     String containing title of plot.
;   plot_cont:  Set to 1 if you want to include countour lines on plot
;
; Color scheme - three-color
;               red   = 1-sigma (Deltachi2 = 1)
;               blue  = 2-sigma (Deltachi2 = 4)
;               green = 3-sigma (Deltachi2 = 9)
;
; Color scheme - Rainbow:
;               red    = 1-sigma (Deltachi2 = 1)
;               orange = 2-sigma (Deltachi2 = 4)
;               yellow = 3-sigma (Deltachi2 = 9)
;               green  = 4-sigma (Deltachi2 = 16)
;               blue   = 5-sigma (Deltachi2 = 25)
;               purple = 6-sigma (Deltachi2 = 36)
;         large black  = 7-sigma (Deltachi2 = 49)
;         small black  > 7 sigma (Deltachi2 > 49)
;
; Written by Gail H. Schaefer.

!P.multi = 0.0

!P.font=0
!P.charsize=1.0
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0
pthick = 4.0

; Set color table
;  red=1, green=2, blue=3

tvlct, [255,0,0], [0,255,0], [0,0,255], 1

; Plot in rainbow colors?
; keyword_set returns True (1) if psf_name is defined, False (0) if undefined

ans_rainbow = keyword_set(rainbow)

if (ans_rainbow) then begin
    loadct,40  ; rainbow+black
    ; Colors:
    ; red = 250, orange=220, yellow=195, green=150, blue=60, purple=30, black=0
    ctab = [250,220,195,150,60,30,0]
endif

; Should we mark position of choice on map?
ans_mark = keyword_set(markpos)
if (ans_mark) then begin
   ra_mark = markpos(0)
   dec_mark = markpos(1)
endif

if (keyword_set(ptitle) ne 1) then ptitle=' '

if (keyword_set(plot_cont) ne 1) then plot_cont=0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

izero = where(chi2_array eq 0.0)
if (izero ge 0) then chi2_array(izero) = max(chi2_array)

size_arr = size(chi2_name)
xsize = size_arr(1)
ysize = size_arr(2)

chimin = min(chi2_array)
print,'chimin before scaling:',chimin

; Scale chi2 so that chired = 1

scale = dof/chimin

chi2_array = chi2_array*scale

chimin = min(chi2_array)

indbest = where(chi2_array eq chimin)

; Separate solutions into 1-, 2-, and 3- sigma intervals

sig1 = where(chi2_array le (chimin+1.0), count1)
sig2 = where(chi2_array gt (chimin+1.0) and chi2_array le (chimin+4.0), count2)
sig3 = where(chi2_array gt (chimin+4.0) and chi2_array le (chimin+9.0), count3)
sig4 = where(chi2_array gt (chimin+9.0) and chi2_array le (chimin+16.0), count4)
sig5 = where(chi2_array gt (chimin+16.0) and chi2_array le (chimin+25.0), count5)
sig6 = where(chi2_array gt (chimin+25.0) and chi2_array le (chimin+36.0), count6)
sig7 = where(chi2_array gt (chimin+36.0) and chi2_array le (chimin+49.0), count7)

sigrem = where(chi2_array gt (chimin+9.0), countrem)
sigrem_rain = where(chi2_array gt (chimin+49.0), countrem_rain)

; 1-sigma interval for two-degrees of freedom
; Delta chi2 = 2.30

;sig2par = where(chi2_array le (chimin+2.3), count2par)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Compute major and minor axis of error ellipse and position angle on sky

ra_best = ra_array(indbest(0))
dec_best = dec_array(indbest(0))

sigma_ra = ra_array(sig1) - ra_best
sigma_dec = dec_array(sig1) - dec_best

;sigma_ra = ra_array(sig2par) - ra_best
;sigma_dec = dec_array(sig2par) - dec_best

radius = sqrt(sigma_ra^2 + sigma_dec^2)
max_radius = max(radius)
min_radius = min(radius)

indmax = where(radius eq max(radius))
indmin = where(radius eq min(radius))

;indmax = indmax(0)
;indmin = indmin(0)

; Create an array of single-values sigma_ra and sigma_dec to define
; contours of error ellipse

sigra_ell = dblarr(count1)
sigdec_ell = dblarr(count1)
n_ell = 0

for i=0, count1-1 do begin

   sra = sigma_ra(i)
   ind_ra = where(sigma_ra eq sra)
   check_dec = sigma_dec(ind_ra)

   sdec = sigma_dec(i)
   ind_dec = where(sigma_dec eq sdec)
   check_ra = sigma_ra(ind_dec)

   if (sra ge max(check_ra)) or (sdec ge max(check_dec)) then begin
      ;print,i,' max max:',sra,sdec
      sigra_ell(n_ell) = sra
      sigdec_ell(n_ell) = sdec
      n_ell = n_ell + 1
   endif else begin
   if (sra le min(check_ra)) or (sdec le min(check_dec)) then begin
      ;print,i,' min min:',sra,sdec
      sigra_ell(n_ell) = sra
      sigdec_ell(n_ell) = sdec
      n_ell = n_ell + 1
   endif
   endelse

endfor
sigra_ell = sigra_ell(0:n_ell-1)
sigdec_ell = sigdec_ell(0:n_ell-1)


radius = sqrt(sigra_ell^2 + sigdec_ell^2)
max_radius = max(radius)
min_radius = min(radius)

indmax = where(radius eq max(radius))
indmin = where(radius eq min(radius))

pa_major = atan(sigra_ell(indmax),sigdec_ell(indmax))
pa_minor = atan(sigra_ell(indmin),sigdec_ell(indmin))

; Compute size and rotation of 1-sigma error ellipse:

params = [max_radius,min_radius,0.0,0.0,pa_major]
npar=n_elements(params)

temp = ellipse_mpfunc(params,x=sigdec_ell,y=sigra_ell)

fa={x:sigdec_ell,y:sigra_ell}

parinfo = replicate({value:0.D, fixed:0, limited:[0,0], $
                                  limits:[0.D,0],step:1e-3,tied:''}, npar)

; Uncomment to fix parameters:
;parinfo(0).fixed=1 ; a
;parinfo(1).fixed=1 ; b
;parinfo(2).fixed=1 ; xcent 
;parinfo(3).fixed=1 ; ycent
;parinfo(4).fixed=1 ; theta

params_ell = mpfit('ellipse_mpfunc', params, functargs=fa,bestnorm=bestnorm,$
           parinfo=parinfo,perror=perror,covar=covar,status=status,xtol=1d-7)


if (n_ell ge 5) then begin

   create_ellipse,xe,ye,1000,major=params_ell(0),minor=params_ell(1),xcent=params_ell(2),ycent=params_ell(3),theta=params_ell(4)*180.0/!dpi,/showplot

   xr = max([params_ell(0),params_ell(1)])

   plot,sigma_ra,sigma_dec,xrange=[1.1*xr,-1.1*xr],yrange=[-1.1*xr,1.1*xr],psym=1,/isotropic
   oplot,sigra_ell,sigdec_ell,psym=2
   oplot,ye,xe,thick=1

endif

if (params_ell(4) lt 0.0) then params_ell(4)=params_ell(4)+!dpi

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Define user symbol to be closed circle
; Make a vector of 16 points, Z[i] = 2pi/16:
Z = findgen(17) * (!pi*2/16.)

fracsmall = 0.25
fracmed = 0.5
frac = 1.0

; Plot orbits as small filled circles
set_plot, 'ps'
device, /Color, filename='temp.eps',xsize=15,ysize=15,/encap


if (keyword_set(xrange)) then begin

   xmin = min(xrange)
   xmax = max(xrange)

   ymin = min(yrange)
   ymax = max(yrange)

endif else begin

   xmin = min(ra_array)
   xmax = max(ra_array)

   ymin = min(dec_array)
   ymax = max(dec_array)

endelse

;print,xmin,xmax
;print,ymin,ymax

if (ans_rainbow) then begin

    ;plot, ra_array, dec_array, psym=8, xtitle="sepRA (mas)", ytitle="sepDEC (mas)",$
    usersym, fracsmall*cos(Z), fracsmall*sin(Z), /fill
    plot, ra_array, dec_array, psym=8, xtitle='!9D!3RA (mas)',ytitle='!9D!3DEC (mas)',$
      /nodata, xstyle=1, ystyle=1, xrange=[xmax,xmin],yrange=[ymin,ymax],/isotropic,$
      title=ptitle;,xtickinterval=0.5,ytickinterval=0.5
    if (countrem_rain gt 0) then oplot, ra_array(sigrem_rain), dec_array(sigrem_rain), psym=8
    ; If overplotting contours, then make plotting symbols smaller 
    if (plot_cont) and (count3 gt 50) then frac = fracmed
    usersym, frac*cos(Z), frac*sin(Z), /fill
    if (count7 gt 0) then oplot, ra_array(sig7), dec_array(sig7), psym=8, color=ctab(6)
    if (count6 gt 0) then oplot, ra_array(sig6), dec_array(sig6), psym=8, color=ctab(5)
    if (count5 gt 0) then oplot, ra_array(sig5), dec_array(sig5), psym=8, color=ctab(4)
    if (count4 gt 0) then oplot, ra_array(sig4), dec_array(sig4), psym=8, color=ctab(3)
    if (count3 gt 0) then oplot, ra_array(sig3), dec_array(sig3), psym=8, color=ctab(2)
    if (count2 gt 0) then oplot, ra_array(sig2), dec_array(sig2), psym=8, color=ctab(1)
    if (count1 gt 0) then oplot, ra_array(sig1), dec_array(sig1), psym=8, color=ctab(0)


    if (plot_cont) then begin

       ; Plot black contours
       ;lev = findgen(21)^2  + chimin
       ;contour,chi2_array,ra_array, dec_array,levels=lev,/overplot

       ; Only plot contours if there are enough points to generate 
       ; reasonable contour surfaces

       if (count3 gt 50) then begin

          ; Plot rainbow contours (1, 2, 3, ... sigma contours)
          lev = [1.0,4.0,9.0,16.0,25.0,36.0,49.0] + chimin
          clev = [250,215,195,150,60,30,0.0]
          nlev = n_elements(lev)
          ; red = 250, orange=220;yellow=195, green=150, blue=60, purple=30
          contour,chi2_array,ra_array, dec_array,levels=lev,c_colors=clev,/overplot,thick=pthick
          ;for ii=0, nlev-1 do contour,chi2_array,ra_array, dec_array,levels=lev(nlev-1-ii),c_colors=clev(nlev-1-ii),/overplot

       endif
    endif

    if (ans_mark) then oplot,[ra_mark],[dec_mark],psym=2,color=ctab(0)

endif else begin

    ;plot, ra_array, dec_array, psym=8, xtitle="sepRA (mas)", ytitle="sepDEC (mas)",$
    plot, ra_array, dec_array, psym=8, xtitle='!9D!3RA (mas)',ytitle='!9D!3DEC (mas)',$
          /nodata, xstyle=1, ystyle=1, xrange=[xmax,xmin],yrange=[ymin,ymax],/isotropic
    ;oplot, ra_array, dec_array, psym=8 ; plot all solutions
    if (count3 gt 0) then oplot, ra_array(sig3), dec_array(sig3), psym=8, color=2
    if (count2 gt 0) then oplot, ra_array(sig2), dec_array(sig2), psym=8, color=3
    if (count1 gt 0) then oplot, ra_array(sig1), dec_array(sig1), psym=8, color=1

    ; Plot black contours
    ;lev = findgen(21)^2  + chimin
    ;contour,chi2_array,ra_array, dec_array,levels=lev,/overplot

    ; Plot red, blue, green contours (1, 2, 3 sigma contours)
    lev = [1.0,4.0,9.0] + chimin
    clev = [1,3,2]
    contour,chi2_array,ra_array, dec_array,levels=lev,c_colors=clev,/overplot

    if (ans_mark) then oplot,[ra_mark],[dec_mark],psym=2,color=1

endelse

if (count1 gt 0) then begin
   usersym, cos(Z), sin(Z), /fill
;   oplot,[sigma_ra(indmax)+ra_best],[sigma_dec(indmax)+dec_best],psym=8
;   oplot,[-sigma_ra(indmax)+ra_best],[-sigma_dec(indmax)+dec_best],psym=8
   usersym, fracsmall*cos(Z), fracsmall*sin(Z), /fill
endif

device,/close
set_plot,'x'


if (count1 gt 0) then begin

   print,'Minimum chi2:',chimin

   print,' '
   print,'---------------------------------------------------------------'
   print,'1-sigma intervals:'
   ;print,'RA min max: ', min(ra_array(sig1)),max(ra_array(sig1))
   ;print,'DEC min max: ', min(dec_array(sig1)),max(dec_array(sig1))
   print,'Best RA: ',ra_array(indbest(0)), ' + ',max(ra_array(sig1))-ra_array(indbest(0)),' - ',ra_array(indbest(0)) - min(ra_array(sig1))
   print,'Best DEC: ',dec_array(indbest(0)), ' + ',max(dec_array(sig1))-dec_array(indbest(0)),' - ',dec_array(indbest(0)) - min(dec_array(sig1))

endif
print,'---------------------------------------------------------------'
print,'Uncertainty ellipse from fitting 1-sigma contours:'
print,'Semi-major axis:',params_ell(0)
print,'Semi-minor axis:',params_ell(1)
print,'sigRA center:',params_ell(3)
print,'sigDEC center:',params_ell(2)
print,'Position angle:',params_ell(4)*180.0/!dpi
print,'---------------------------------------------------------------'


if (count1 gt 0) then begin
   print,' '
   print,'Major axis of error ellipse (size, PA): ',max_radius,PA_major*180.0/!dpi
   print,'Minor axis of error ellipse (size, PA): ',min_radius,PA_minor*180.0/!dpi
endif

;stop,'Type .cont to continue'

end
