pro auto_plot_model_bline_6T, v2data, t3data, oiarray, vis2model, t3model, infile, vis2model2=vis2model2, t3model2=t3model2

; Plot visibilities and closure phases for MIRC 6T data.
; Overplot best fit model.
; No grouping of baselines.
;
; Input Parameters:
;   v2data    - structure containing visibility amplitude information.
;   t3data    - structure containing triple product information.
;   oiarray   - structure containing OI-data information.
;   vis2model - structure containing model visibilities at same uv points
;               as v2data  
;   t3model   - structure containing model triple products at same uv points
;               as t3data  
;   infile    - name of original OIFITS file used to extract v2data and
;               t3data structures.  Used when saving ouput plots.
;
; Optional Input Parameters:
;   vis2model2 - second structure containing visibilities of a second model
;                at same uv points as v2data
;   t3model2   - second structure containing triple products of a second model
;                at same uv points as t3data  
;
; OUTPUT:
; The following files are created by this routine:
;   filename_t3_bline.eps
;   filename_t3.eps
;   filename_uv_lam.eps
;   filename_uv.eps
;   filename_vis2_big.eps
;   filename_vis2.eps
; where filename is the base name of the oifits file
;
; Written by Gail H. Schaefer.  Based, in part, on a routine originally
;   written by John Monnier.
;
; Revision history:
; 21 Apr 2016 - GHS: allow option of plotting a second model.
; 15 Nov 2011 - GHS: modified for 6T.
; 05 Aug 2009 - GHS: created program to plot vis with models.
; 05 Aug 2009 - GHS: edited 4t_detail.script to save plots to separate files.


; Is a second model defined?
; keyword_set returns True (1) if vis2model2 is defined, False (0) if undefined
ans_plot2 = keyword_set(vis2model2)

loadct,0

; These are actually set below!!!!
!P.charsize=0.8
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0

!p.multi=0 ; 

num_v2 = n_elements(v2data.vis2data) 
num_t3 = n_elements(t3data.t3phi)

;base_names_arr = oiarray[v2data.sta_index].sta_name
;tri_names_arr = oiarray[t3data.sta_index].sta_name

v2_plotted = intarr(n_elements(v2data)) ; used to keep track of plotted data.
t3_plotted = intarr(n_elements(t3data))

;Split into various Baselines and TIMES.
v2_index=v2data.sta_index[0]*10+v2data.sta_index[1]
t3_index=t3data.sta_index[0]*100+t3data.sta_index[1]*10+t3data.sta_index[2]

uniq_base=v2_index(uniq(v2_index,sort(v2_index)))
uniq_tri =t3_index(uniq(t3_index,sort(t3_index)))
ntri=n_elements(uniq_tri)
nbase=n_elements(uniq_base)

; Do naming
basenames = strarr(nbase)
pa_ave = dblarr(nbase)

for b=0,nbase-1 do begin
  in = where(v2_index eq uniq_base(b))
  in0=(where(v2_index eq uniq_base(b)))(0)
  sta_index= v2data(in0).sta_index
  if keyword_set(oiarray) and v2data(in0).arrname ne 'UVHOLE' then begin ; name the triangles too if oiarray exists
    vin0=(where(oiarray.sta_index eq sta_index(0)))(0)
    vin1=(where(oiarray.sta_index eq sta_index(1)))(0)

   basenames(b)=strcompress( oiarray(vin0).sta_name+'-'+oiarray(vin1).sta_name,/rem)

   pa_ave(b) = mean(v2data(in).PA)

   endif else basenames(b) = strcompress( string(uniq_base(b)),/rem)
   if v2data(in0).arrname eq 'UVHOLE' then basenames(b) = 'UVHOLE'
endfor

trinames = strarr(ntri)
for t=0,ntri-1 do begin
  in0=where(t3_index eq uniq_tri(t))
  sta_index= t3data(in0).sta_index  
  if keyword_set(oiarray) then begin ; name the triangles too if oiarray exists
    tin0=(where(oiarray.sta_index eq sta_index(0)))(0)
    tin1=(where(oiarray.sta_index eq sta_index(1)))(0)
    tin2=(where(oiarray.sta_index eq sta_index(2)))(0)

   trinames(t) = strcompress( oiarray(tin0).sta_name+'-'+oiarray(tin1).sta_name+'-'+$
                          oiarray(tin2).sta_name,/rem )
   endif else trinames(t) = strcompress( string(uniq_tri(t)),/rem)
endfor

;stop,'Type .cont to continue'


; Compute chi2 for V2 and T3 data separately

chi2_all = total([ (v2data.vis2data - vis2model.vis2data)^2/(v2data.vis2err)^2,$
              (angle_diff(t3data.t3phi,t3model.t3phi))^2/(t3data.t3phierr)^2])

chi2_v2 = total([ (v2data.vis2data - vis2model.vis2data)^2/(v2data.vis2err)^2])

chi2_t3 = total([(angle_diff(t3data.t3phi,t3model.t3phi))^2/(t3data.t3phierr)^2])

num_v2 = n_elements(v2data.vis2data)
num_t3 = n_elements(t3data.t3phi)
num_all = num_v2 + num_t3

redchi_all = chi2_all/num_all
redchi_v2 = chi2_v2/num_v2
redchi_t3 = chi2_t3/num_t3

; compare with bestnorm:
;print,'All chi2:',chi2_all,num_all,redchi_all
;print,'V2 chi2:',chi2_v2,num_v2,redchi_v2
;print,'T3 chi2:',chi2_t3,num_t3,redchi_t3

; Make images of the model:

; Remove directory prefix and oifits extension
infile_sep = strsplit(infile,'/',/extract)
nz = n_elements(infile_sep)
infilename=infile_sep(nz-1)

nz1 = strlen(infilename) - strlen('.oifits')
nz2 = strlen(infilename)
check = strmid(infilename,nz1,nz2)
if (check eq '.oifits') then pfname = strmid(infilename,0, nz1) $
else begin
   nz1 = strlen(infilename) - strlen('.fits')
   pfname = strmid(infilename,0, nz1)
endelse
print,pfname

pfname_sep = strsplit(pfname,'.',/extract)
pflabel = pfname_sep(0)
ptitle = pflabel

;ptitle=''
;print,'Enter date label for the plots (e.g. 2007Nov14+19):'
;read,ptitle
ptitle = v2data(0).date_obs
ptlen = strlen(ptitle)
;pflabel=ptitle

;stop,'Type .cont to continue'


;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


cleanplot

; Create plots

set_plot,'ps'

!p.multi=0

; Set color table
;  red=1, green=2, blue=3
tvlct, [255,0,0], [0,255,0], [0,0,255], 1

;loadct,10
;!p.charsize=1
;blsize=0.75
;pcsize = 1.0
blsize=1.0;0.5
pcsize = 0.8
ys = 0.05;0.05 ; 0.03
;!P.charsize=0.8
!P.charsize=1.0
;!P.charsize=1.4
!P.charthick=3.0
!P.thick=3.0
!X.thick=4.0
!Y.thick=4.0
if (!d.name eq 'X') then begin
 window,0,xsize=800,ysize=500
endif

;device,filename='temp_uv.ps',xsize=8,ysize=8
;device,filename=pfname+'_uv.eps',xsize=8,ysize=8,/portrait,/encap
device,filename=pfname+'_uv.eps',xsize=12,ysize=12,/portrait,/encap

; First plot UV Coverage and Vis2 data.
xv=[350,-350]
yv=[-350,350]

;xv=[200,-200]
;yv=[-200,200]

plot,[v2data(in).ucoord],[v2data(in).vcoord],psym=8,/nodata, $
 xtitle="East (m)",ytitle="North (m)",xrange=xv,yrange=yv,xstyle=1,ystyle=1,/isotropic

syms=[0,3,4,5,8,0,3,4,5,8,0,3,4,5,8,0,3,4,5,8,0,3,4,5,8,0,3,4,5,8,0,3,4,5,8,0,3,4,5,8]
fills=[1,1,1,1,00,0,0,0,00,1,1,0,0,0,1,1,1,1,1,1,1,1,00,0,0,0,00,1,1,0,0,0,1,1,1,1]
for i=0,nbase-1 do begin
 in = where(v2_index eq uniq_base(i))
 plotsym,syms(i),fill=fills(i)
 oplot,[v2data(in).ucoord],[v2data(in).vcoord],psym=8,symsize=1.2
 oplot,[-1*v2data(in).ucoord],[-1*v2data(in).vcoord],psym=8,symsize=1.2
 plots,xv(0)+(xv(1)-xv(0))*.05,$
       yv(0)+(yv(1)-yv(0))*(.95-i*ys),psym=8
 xyouts,xv(0)+(xv(1)-xv(0))*.09,$
       yv(0)+(yv(1)-yv(0))*(.94-i*ys),basenames(i),charsize=blsize

endfor
oplot,[0],[0],psym=1
;;;;xyouts,xv(0)+(xv(1)-xv(0))*.05,yv(0)+(yv(1)-yv(0))*.05,pflabel,charsize=pcsize
;xyouts,xv(0)+(xv(1)-xv(0))*.05,yv(0)+(yv(1)-yv(0))*.05,strmid(infile,strlen(imdir)+strlen('OIFITS/'), 9),charsize=pcsize
;xyouts,300,-300,'2007Nov11-14',charsize=pcsize

;xyouts,.1,.92,'DATAFILE: '+strmid(infile, strpos(infile, 'oifits')-48, 55),charsize=1,/norm

device,/close

;;------------------------------------------------------------------------
; make uv plot - in mega-lambda units

;device,filename='temp_uv_lam.ps',xsize=8,ysize=8
;device,filename=pfname+'_uv_lam.eps',xsize=8,ysize=8,/encap
device,filename=pfname+'_uv_lam.eps',xsize=12,ysize=12,/encap

xvp=[250,-250]
yvp=[-250,250]

!P.charsize=1.0
!P.charthick=3.0
!P.thick=3.0
!X.thick=3.0
!Y.thick=3.0

; Define user symbol to be closed circle
; Make a vector of 16 points, Z[i] = 2pi/16:
Z = findgen(17) * (!pi*2/16.)

frac = 0.25    ;scale factor for adjusting size of plotting symbols

;Define the symbol to be a unit circle with 16 points
usersym, frac*cos(Z), frac*sin(Z), /fill


plot,[v2data(in).u/1e6],[v2data(in).v/1e6],psym=8,/nodata, $
; xtitle="u x10!u-6!n (cycles/radian)",ytitle="v x10!u-6!n (cycles/radian)",charsize=0.8, $
 xtitle="u (10!u6!n cycles/radian)",ytitle="v (10!u6!n cycles/radian)", $
 xrange=xvp,yrange=yvp,xstyle=1,ystyle=1,/isotropic


for i=0,nbase-1 do begin
 in = where(v2_index eq uniq_base(i))
 ;plotsym,syms(i),fill=fills(i)
 oplot,[v2data(in).u/1e6],[v2data(in).v/1e6],psym=8
 oplot,[-1*v2data(in).u/1e6],[-1*v2data(in).v/1e6],psym=8
; plots,xvp(0)+(xvp(1)-xvp(0))*.05,$
;       yvp(0)+(yvp(1)-yvp(0))*(.96-i*ys),psym=8
; xyouts,xvp(0)+(xvp(1)-xvp(0))*.09,$
;       yvp(0)+(yvp(1)-yvp(0))*(.95-i*ys),basenames(i),charsize=blsize
endfor
oplot,[0],[0],psym=1

; UNDELETE IF YOU WANT TO LABEL PLOT WITH THE FILE NAME
;xyouts,xvp(0)+(xvp(1)-xvp(0))*.05,yvp(0)+(yvp(1)-yvp(0))*.05,pflabel,charsize=pcsize
; UNDELETE IF YOU WANT TO LABEL PLOT WITH USER DEFINED LABEL
;xyouts,xvp(1)-(xvp(1)-xvp(0))*.05,yvp(1)-(yvp(1)-yvp(0))*.08,ptitle,alignment=1.0

device,/close

;------------------------------------------------------------------------
;make big vis plot

; Set color table
;  red=1, green=2, blue=3
tvlct, [255,0,0], [0,255,0], [0,0,255], 1

device,filename=pfname+'_vis2_big.eps',xsize=17,ysize=12,/color, Bits_per_pixel=8,/encap  ; 1 plot of 12x8

!p.multi=0 ; one big plot for this version of plot.

xr=[0,max(v2data.sfu)]/1e6
;xr=[0,250.0]
yr=[0 < (min(v2data.vis2data)),1.0 > max(v2data.vis2data)*1.1]
;yr=[0, 1.0]

;xv=[350,-350]
;yv=[-350,350]


uvmax=max([abs(v2data.u),abs(v2data.v)])
xv=[uvmax,-uvmax]
yv=[-uvmax,uvmax]


; Make one big plot of all visibilities!!!

!P.charsize=1.2  

plot,v2data.sfu/1e6,v2data.vis2data,xrange=xr,yrange=yr,xtitle="!3B/!4k!3 x10!u-6!n", ytitle="V!u2!n",/nodata,ystyle=1


; Plot measured values
; Define the symbol to be a circle
frac = 1.0
usersym, frac*cos(Z), frac*sin(Z), /fill
oploterror,v2data.sfu/1e6,v2data.vis2data,v2data.vis2err,psym=8,errthick=1

; Overplot model vis2 at exact values of measured spatial frequencies
; as red crosses
oplot,[vis2model.sfu/1e6],[vis2model.vis2data],psym=7,symsize=frac,color=1,thick=3.0

if (ans_plot2) then oplot,[vis2model2.sfu/1e6],[vis2model2.vis2data],psym=7,symsize=frac,color=3,thick=3.0

device,/close

;------------------------------------------------------------------------
;make big vis plot - log scaling

; Set color table
;  red=1, green=2, blue=3
tvlct, [255,0,0], [0,255,0], [0,0,255], 1

device,filename=pfname+'_logvis2_big.eps',xsize=17,ysize=12,/color, Bits_per_pixel=8,/encap  ; 1 plot of 12x8

!p.multi=0 ; one big plot for this version of plot.

xr=[0,max(v2data.sfu)]/1e6
;xr=[0,250.0]
;yr=[0 < (min(v2data.vis2data)),1.0 > max(v2data.vis2data)*1.1]
yr=[1e-6 > (min(v2data.vis2data)*0.1),1.0 > max(v2data.vis2data)*1.1]
;yr=[0, 1.0]

;xv=[350,-350]
;yv=[-350,350]


uvmax=max([abs(v2data.u),abs(v2data.v)])
xv=[uvmax,-uvmax]
yv=[-uvmax,uvmax]


; Make one big plot of all visibilities!!!

!P.charsize=1.2  

plot,v2data.sfu/1e6,v2data.vis2data,xrange=xr,xtitle="!3B/!4k!3 x10!u-6!n", ytitle="V!u2!n",/nodata,/ylog,ystyle=1,yrange=yr


; Plot measured values
; Define the symbol to be a circle
frac = 1.0
usersym, frac*cos(Z), frac*sin(Z), /fill
oploterror,v2data.sfu/1e6,v2data.vis2data,v2data.vis2err,psym=8,errthick=1

; Overplot model vis2 at exact values of measured spatial frequencies
; as red crosses
oplot,[vis2model.sfu/1e6],[vis2model.vis2data],psym=7,symsize=frac,color=1,thick=3.0

if (ans_plot2) then oplot,[vis2model2.sfu/1e6],[vis2model2.vis2data],psym=7,symsize=frac,color=3,thick=3.0

device,/close

;------------------------------------------------------------------------
;make big residual vis plot

; Set color table
;  red=1, green=2, blue=3
tvlct, [255,0,0], [0,255,0], [0,0,255], 1

device,filename=pfname+'_vis2_resid.eps',xsize=17,ysize=12,/color, Bits_per_pixel=8,/encap  ; 1 plot of 12x8

!p.multi=0 ; one big plot for this version of plot.

diff = v2data.vis2data - vis2model.vis2data

xr=[0,max(v2data.sfu)]/1e6
;xr=[0,250.0]
yr=[-max(abs(diff)),max(abs(diff))]*1.1
;yr=[0, 1.0]

;xv=[350,-350]
;yv=[-350,350]


; Make one big plot of all visibility residuals!!!

!P.charsize=1.2  

plot,v2data.sfu/1e6,diff,xrange=xr,yrange=yr,xtitle="!3B/!4k!3 x10!u-6!n", ytitle="V!u2!n (data - model)",/nodata,ystyle=1


; Plot measured values
; Define the symbol to be a circle
frac = 1.0
usersym, frac*cos(Z), frac*sin(Z), /fill
oploterror,v2data.sfu/1e6,diff,v2data.vis2err,psym=8,errthick=1

device,/close

;------------------------------------------------------------------------
;make vis plot

;set_plot,'x'
;device,filename='temp_vis2.ps',xsize=12,ysize=8
;device,filename=pfname+'_vis2.ps',xsize=12,ysize=8 ; 1 plot of 12x8
;device,filename=pfname+'_vis2.ps',xsize=18,ysize=8 ; 3 by 2 plots of 6x4
;device,filename=pfname+'_vis2.ps',xsize=18,ysize=12 ; 2 by 2 plots of 9x6
;device,filename=pfname+'_vis2.ps',xsize=24,ysize=22,/color

; For 6T data, plot 15 baselines on 4x4 grid.  Modify file size for < 6T.
yp = nbase/4.0
ypf = fix(yp)
if (ypf ne yp) then ypf = ypf + 1.0
if (nbase lt 4.0) then xpf = nbase else xpf = 4.0
xpf = fix(xpf)
ypf = fix(ypf)

if (nbase le 4.0) then device,filename=pfname+'_vis2.eps',xsize=1.2*(24.0/4.0)*xpf,ysize=1.2*(22.0/4.0)*ypf,/color,/encap $
else device,filename=pfname+'_vis2.eps',xsize=(24.0/4.0)*xpf,ysize=(22.0/4.0)*ypf,/color,/encap

;print,'YES!!!!!!!!!!!!!!!!!!!'

!p.multi=0 ; one big plot for this version of plot.

if (!d.name eq 'X') then begin
 window,1,xsize=800,ysize=500
endif

multiplot,/reset
cleanplot
;!p.multi=[0,2,2]
;!p.multi=[0,4,6]
;!p.multi=[0,4,4]
!p.multi=[0,xpf,ypf]
;if !d.name eq 'X' then !p.charsize=2
;if !d.name eq 'PS' then !p.charsize=1.3
!p.charsize=0.8
!P.charthick=3.0
!P.thick=3.0
!X.thick=3.0
!Y.thick=3.0

; Set color table
;  red=1, green=2, blue=3
tvlct, [255,0,0], [0,255,0], [0,0,255], 1

vis2_arr = v2data.vis2data
vis2err_arr = v2data.vis2err
u_arr = v2data.u
v_arr = v2data.v
ucoord = v2data.ucoord
vcoord = v2data.vcoord
bline = v2data.baseline
wave = v2data.eff_wave
sfu = v2data.sfu

; Note the following routine is the same as doing 
;   sfu = v2data(0).BASELINE/v2data(0).EFF_WAVE
;ri2at,v2data.u,v2data.v,sfu,theta
;vis=sign(v2data.vis2data)*sqrt( abs(v2data.vis2data))
;viserr=abs(vis* v2data.vis2err/v2data.vis2data)*.5

;xlab1=['0','50','100','150','200',' ']
;xlab2=[' ','50','100','150','200',' ']
;xlab3=[' ','50','100','150','200','250']
;xlab0=[' ',' ',' ',' ',' ',' ']
;
;ylab1=['0.0','0.2','0.4','0.6','0.8','1.0']
;ylab0=[' ',' ',' ',' ',' ',' ']
;psz = 0.7  ; symsize 3x2
;;psz = 1.0  ; symsize 2x2

 ;xr=[0,max(sfu)]/1e6
xr_def=[0.8*min(sfu),1.2*max(sfu)]/1e6
;xr_def=[0,250.0]
;xr=[min(sfu),max(sfu)]/1e6
;yr=[0 < (min(vis)),max(vis)*1.2]
;yr=[0 < (min(v2data.vis2data)),1.0 > max(v2data.vis2data)*1.1]
yr=[0.0,1.0 > max(v2data.vis2data)*1.1]
;yr_def=[0, 1.0]


gi = 0.0   ; count for grouping plots on a page
rowi = 0.0
   
;!P.charsize=1.1  ;3x2
;!P.charsize=1.2   ;2x2
;!P.charsize=1.1   ; 1 plot per page

ans_cont = ''

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Plot 4x4 per page
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
for i=0,nbase-1 do begin

   if (i+4 ge nbase) then xlab="!3B/!4k!3 x10!u-6!n" else xlab=''
   
  in = where(v2_index eq uniq_base(i))
;  xr_def=round([min(sfu(in))/1e6 - 10,max(sfu(in))/1e6 + 10]/10.0)*10
  xdiff = (max(sfu(in)) - min(sfu(in)))/1e6
  xr_def=[min(sfu(in))/1e6 - 0.20*xdiff,max(sfu(in))/1e6 + 0.20*xdiff]
  if (xdiff lt 1.0) then xr_def=[0.8*min(sfu(in)),1.2*max(sfu(in))]/1e6

  ;print,xr_def

  ; Define the symbol to be a circle
  frac = 1.0
  usersym, frac*cos(Z), frac*sin(Z), /fill
 
if (rowi eq 0) then begin
 
    if (gi eq 0) then begin
       erase
       multiplot,/reset
       ;multiplot,[4,4],/init,ygap=0.02,/doxaxis
       multiplot,[xpf,ypf],/init,ygap=0.02,/doxaxis
       multiplot,/doxaxis
;       yr=[yr_def(0)+0.00001,yr_def(1)]
       xr=[xr_def(0),xr_def(1)-0.001]    
       ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
       plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           ytit='V!u2!n',xtit=xlab,/nodata
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ;if (gi eq 1) then xr(0)=90.001
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab,/nodata
    endif

    if (gi eq 3) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab,/nodata
    endif
endif
if (rowi eq 1) then begin
 
    if (gi eq 0) then begin
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0),xr_def(1)-0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab, $
           ytit='V!u2!n',/nodata
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab,/nodata
    endif

    if (gi eq 3) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab,/nodata
    endif
endif
if (rowi eq 2) then begin
 
    if (gi eq 0) then begin
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0),xr_def(1)-0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab, $
           ytit='V!u2!n',/nodata
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab,/nodata
    endif

    if (gi eq 3) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0)+0.001,yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           xtit=xlab,/nodata
    endif
endif
if (rowi eq 3) then begin
 
    if (gi eq 0) then begin
       multiplot,/doxaxis
;       yr=[yr_def(0),yr_def(1)-0.001]
       xr=[xr_def(0),xr_def(1)-0.00001]    
       ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
       plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           ytit='V!u2!n', xtit=xlab,/nodata
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0),yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           xtit=xlab ,/nodata   
    endif

    if (gi eq 3) then begin 
      multiplot,/doxaxis
;      yr=[yr_def(0),yr_def(1)-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ;ploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),$
      plot,sfu(in)/1e6,vis2_arr(in),$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           xtit=xlab,/nodata    
    endif
endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;xyouts,xr_def(0)+0.05*(xr_def(1) - xr_def(0)),0.90,basenames(i),alignment=0.0,charsize=0.9

;xyouts,xr_def(0)+0.05*(xr_def(1) - xr_def(0)),0.10,basenames(i),alignment=0.0,charsize=0.9
xyouts,xr_def(0)+0.08*(xr_def(1) - xr_def(0)),0.85,basenames(i),alignment=0.0,charsize=0.9

;xyouts,xr_def(0)+0.5*(xr_def(1) - xr_def(0)),0.85,basenames(i),alignment=0.5,charsize=0.9

;print,basenames(i),ucoord(in(0)),vcoord(in(0))

if (yr(1) gt 1.0) then oplot,xr,[1.0,1.0],linestyle=1


; Plot measured values
usersym, frac*cos(Z), frac*sin(Z), /fill
oploterror,sfu(in)/1e6,vis2_arr(in),vis2err_arr(in),psym=8

; Overplot model vis2 at exact values of measured spatial frequencies
; as red crosses
oplot, [vis2model(in).sfu/1e6],[vis2model(in).vis2data],psym=7,symsize=frac,color=1,thick=3.0

; Plot second model if defined
if (ans_plot2) then oplot, [vis2model2(in).sfu/1e6],[vis2model2(in).vis2data],psym=1,symsize=frac,color=3,thick=3.0

gi = gi + 1
if (gi gt 3.0) then begin
   gi = 0.0
   rowi = rowi + 1
endif

endfor


device,/close

;----------------------------------------------

;device,filename='temp_t3.ps',xsize=12,ysize=8
;device,filename=pfname+'_t3.ps',xsize=12,ysize=8
;device,filename=pfname+'_t3.ps',xsize=30,ysize=22  ; 5x5
;device,filename=pfname+'_t3.ps',xsize=24,ysize=22  ; 5x5

; For 6T data, plot 20 baselines on 4x5 grid.  Modify file size for < 6T.
yp = ntri/4.0
ypf = fix(yp)
if (ypf ne yp) then ypf = ypf + 1.0
if (ntri lt 4.0) then xpf = ntri else xpf = 4.0
xpf = fix(xpf)
ypf = fix(ypf)
if (ntri le 4.0) then device,filename=pfname+'_t3.eps',xsize=(24.0/4.0)*xpf,ysize=1.5*(22.0/5.0)*ypf,/color,/encap $ ; 4x5
else device,filename=pfname+'_t3.eps',xsize=(24.0/4.0)*xpf,ysize=(22.0/5.0)*ypf,/color,/encap  ; 4x5

if (!d.name eq 'X') then begin
window,3,xsize=800,ysize=500
endif

multiplot,/reset
cleanplot
;!p.multi=[0,2,2]
;!p.multi=[0,ntri,2]
;!p.multi=[0,4,5]
!p.multi=[0,xpf,ypf]
;if !d.name eq 'X' then !p.charsize=2
;if !d.name eq 'PS' then !p.charsize=1.3
!p.charsize=0.8
!P.charthick=3.0
!P.thick=3.0
!X.thick=4.0
!Y.thick=4.0

l=[min(t3data.eff_wave),max(t3data.eff_wave)]

yrmax=(max(abs(t3data.t3phi))*1.2)<180.
;yrmax = 180.0
;yrmax = 20.0
;yrmax = 10.0
yr=[-yrmax,yrmax]
;xr=[1.50,1.80]
;xr=[1.48,1.78]    
;xr_def=[1.47,1.78]  ; MIRC
wmin = round(1d6*min(t3data.eff_wave)*100.0)/100.0
wmax = round(1d6*max(t3data.eff_wave)*100.0)/100.0
wr = round(0.10*(wmax - wmin)*100.0)/100.0
if (wr eq 0) then wr = 0.1
xr_def=[wmin-wr,wmax+wr]

gi = 0.0   ; count for grouping plots on a page
rowi = 0.0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Plot 4x5 per page
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
for i=0,ntri-1 do begin

   if (i+4 ge ntri) then xlab='Wavelength (!4l!3m)' else xlab=''

    in=where(t3_index eq uniq_tri(i))
    sta_index=t3data(in(0)).sta_index
    tritit=trinames(i)

    t3phi_data = [t3data(in).t3phi]
    t3phierr_data = [t3data(in).t3phierr]
    eff_wave_data = [t3data(in).eff_wave]

     ;print,i,gi,rowi

    ;multiplot

    ;plotsym,syms(i),fill=fills(i)
    ;Define the symbol to be a circle
    frac = 1.0
    usersym, frac*cos(Z), frac*sin(Z), /fill

if (rowi eq 0) then begin
 
    if (gi eq 0) then begin
       erase
       multiplot,/reset
       ;multiplot,[4,5],/init
       multiplot,[xpf,ypf],/init
       multiplot
       yr=[-yrmax+0.001,yrmax]
       xr=[xr_def(0),xr_def(1)-0.00001]    
;       xr=[1.47,1.77-0.00001]    
       ploterror,1e6*eff_wave_data, t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab, $
           ytit='Closure Phase ('+string("260B)+')'   ;"
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot
      yr=[-yrmax+0.001,yrmax]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
;      xr=[1.47+0.00001,1.77-0.00001]    
      ploterror,1e6*eff_wave_data, t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab
    endif

    if (gi eq 3) then begin 
      multiplot
      yr=[-yrmax+0.001,yrmax]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ploterror,1e6*eff_wave_data, t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab
    endif
endif
if (rowi eq 1) or (rowi eq 2) or (rowi eq 3) then begin
 
    if (gi eq 0) then begin
       multiplot
       yr=[-yrmax+0.001,yrmax-0.001]
       xr=[xr_def(0),xr_def(1)-0.00001]    
       ploterror,1e6*eff_wave_data, t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab, $
           ytit='Closure Phase ('+string("260B)+')' ;"
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot
      yr=[-yrmax+0.001,yrmax-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ploterror,1e6*eff_wave_data, t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab
    endif


    if (gi eq 3) then begin 
      multiplot
      yr=[-yrmax+0.001,yrmax-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ploterror,1e6*eff_wave_data,t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1,xtit=xlab  

    endif
endif
if (rowi eq 4) then begin

    if (gi eq 0) then begin
       multiplot
       yr=[-yrmax,yrmax-0.001]
       xr=[xr_def(0),xr_def(1)-0.00001]    
       ploterror,1e6*eff_wave_data,t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           ytit='Closure Phase ('+string("260B)+')', $ ;"
           xtit=xlab    
    endif 

    if (gi eq 1) or (gi eq 2) then begin 
      multiplot
      yr=[-yrmax,yrmax-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)-0.00001]    
      ploterror,1e6*eff_wave_data,t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           xtit=xlab
    endif

    if (gi eq 3) then begin 
      multiplot
      yr=[-yrmax,yrmax-0.001]
      xr=[xr_def(0)+0.00001,xr_def(1)+0.00001]    
      ploterror,1e6*eff_wave_data,t3phi_data,t3phierr_data,$
           psym=8, yr=yr, xr=xr,yst=1,xst=1, $
           xtit=xlab
    endif
endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;       yr=[-yrmax+0.001,yrmax]
;       xr=[1.50,1.80-0.00001]    
;       yr = [-180.0,180.0]
;       xr=[1.48,1.78]    
;       plot,1e6*t3data(in).eff_wave, t3data(in).t3phi,$
;           psym=8, yrange=yr, xrange=xr, xstyle=1,ystyle=1,/nodata, $ 
;           xtitle='Wavelength (!4l!3m)', $
;           ytitle='Closure Phase ('+string("260B)+')' 
;
;    oploterror,1e6*t3data(in).eff_wave, t3data(in).t3phi,t3data(in).t3phierr,psym=8
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    oplot,[0,1e6],[0,0],line=2
    xyouts,mean(xr_def),yrmax-0.3*yrmax,tritit,charsize=1.0,alignment=0.5
    ;xyouts,mean(xr_def),-yrmax+0.1*yrmax,tritit,charsize=1.0,alignment=0.5
    ;xyouts,mean(xr_def),-yrmax+0.2*yrmax,tritit,charsize=1.0,alignment=0.5 

    ; for sigma ori, move some labels to bottom of plot
    ;if (min(t3phi_data) lt -90.0) then xyouts,mean(xr_def),yrmax-0.3*yrmax,tritit,charsize=1.0,alignment=0.5 $
    ;else xyouts,mean(xr_def),-yrmax+0.2*yrmax,tritit,charsize=1.0,alignment=0.5
    ;if (rowi eq 0) then xyouts,mean(xr_def),yrmax-0.3*yrmax,tritit,charsize=1.0,alignment=0.5 $
    ;else xyouts,mean(xr_def),-yrmax+0.2*yrmax,tritit,charsize=1.0,alignment=0.5

    ; Overplot model t3 at exact values of measured spatial frequencies
    ; as red crosses
    oplot, [1e6*t3model(in).eff_wave],[t3model(in).t3phi],psym=7,symsize=frac,color=1,thick=3.0

    ; Overplot second model t3 at exact values of measured spatial frequencies
    ; as blue plus signs
    if (ans_plot2) then oplot, [1e6*t3model2(in).eff_wave],[t3model2(in).t3phi],psym=1,symsize=frac,color=3,thick=3.0

    ;print,tritit,t3data(in(0)).u1coord,t3data(in(0)).v1coord,t3data(in(0)).u2coord,t3data(in(0)).v2coord,t3data(in(0)).u3coord,t3data(in(0)).v3coord


gi = gi + 1
if (gi gt 3.0) then begin
   gi = 0.0
   rowi = rowi + 1
endif

endfor
l=[min(t3data.eff_wave),max(t3data.eff_wave)]

device,/close

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Plot all closure phases on one plot

multiplot,/reset
cleanplot
!p.charsize=1.0
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0

sfu1 = sqrt((t3data.u1)^2 + (t3data.v1)^2)
sfu2 = sqrt((t3data.u2)^2 + (t3data.v2)^2)
sfu3 = sqrt((t3data.u3)^2 + (t3data.v3)^2)

max_sfu = dblarr(num_t3)

for i=0,num_t3-1 do max_sfu(i) = max([sfu1(i),sfu2(i),sfu3(i)])

xr=[0.8*min(max_sfu),1.2*max(max_sfu)]/1e6
;xr=[0,1.2*max(sfu)]/1e6
;xr(1) = round(xr(1))
;xr=[100,240.0]

yrmax=(max(abs(t3data.t3phi))*1.2)<180.
;yrmax = 180.0
;yrmax = 20.0
;yrmax = 10.0
yr=[-yrmax,yrmax]

; To display the degree symbol in most Postscript fonts (glyph B0 in hex), 
; you have to set the /ISOLATIN1 keyword via the DEVICE command.


;device,filename=pfname+'_t3_bline.ps',xsize=16,ysize=11,/color,/ISOLATIN1  ; 1 plot of 12x8
;device,filename=pfname+'_t3_bline.ps',xsize=10,ysize=7.0,/color,font_size=7, Bits_per_pixel=8,/ISOLATIN1 
;device,filename=pfname+'_t3_bline.eps',xsize=10,ysize=7.2,/color, Bits_per_pixel=8, /ISOLATIN1, /encap
device,filename=pfname+'_t3_bline.eps',xsize=17,ysize=12,/color, Bits_per_pixel=8, /ISOLATIN1, /encap

ploterror,max_sfu/1e6, t3data.t3phi,t3data.t3phierr,$
          psym=8, yr=yr, xr=xr,yst=1,xst=1,position=[0.14,0.13,0.97,0.96], $
 ;         ytit='Closure Phase (!9%!3)', $   ;"
         ytit='Closure Phase ('+string(176b)+')', $   ;"
;          ytit='Closure Phase ('+string("260B)+')', $   ;"
          xtit="Maximum B/!4k!3 x10!u-6!n within Triangle",/nodata,errthick=1
;          xtit="Maximum !3B/!9l!3 x10!u-6!n within Triangle",/nodata,errthick=1
;          xtit="Maximum !3B/!4k!3 x10!u-6!n within Triangle",/nodata,errthick=1
;xtit="!3B/!9l!3 x10!u-6!n"
oplot,xr,[0,0],linestyle=2

frac = 1.0
usersym, frac*cos(Z), frac*sin(Z), /fill

oploterror,max_sfu/1e6, t3data.t3phi,t3data.t3phierr, psym=8
oplot,[max_sfu/1e6], [t3model.t3phi], psym=7,symsize=frac,color=1,thick=3.0

;xl = 7
;for i=0,ntri-1 do begin
;    in=where(t3_index eq uniq_tri(i))
;    oplot,max_sfu(in)/1e6, t3data(in).t3phi, psym=8 ;,color=ctab(i)
;endfor

print,'Min/Max CP:',min(t3data.t3phi),max(t3data.t3phi)

device,/close

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Plot all closure phase residuals on one plot

multiplot,/reset
cleanplot
!p.charsize=1.0
!P.charthick=2.0
!P.thick=2.0
!X.thick=2.0
!Y.thick=2.0

sfu1 = sqrt((t3data.u1)^2 + (t3data.v1)^2)
sfu2 = sqrt((t3data.u2)^2 + (t3data.v2)^2)
sfu3 = sqrt((t3data.u3)^2 + (t3data.v3)^2)

max_sfu = dblarr(num_t3)

for i=0,num_t3-1 do max_sfu(i) = max([sfu1(i),sfu2(i),sfu3(i)])

xr=[0.8*min(max_sfu),1.2*max(max_sfu)]/1e6
;xr=[0,1.2*max(sfu)]/1e6
;xr(1) = round(xr(1))
;xr=[100,240.0]

diff = t3data.t3phi - t3model.t3phi

nz = n_elements(diff)
for i=0, nz-1 do begin
   if (diff(i) gt 180.0) then diff(i) = diff(i) - 360.0
endfor
 for i=0, nz-1 do begin
   if (diff(i) lt -180.0) then diff(i) = diff(i) + 360.0
endfor
   
yrmax=max(abs(diff))
;yrmax = 180.0
;yrmax = 20.0
;yrmax = 10.0
yr=[-yrmax,yrmax]

; To display the degree symbol in most Postscript fonts (glyph B0 in hex), 
; you have to set the /ISOLATIN1 keyword via the DEVICE command.


device,filename=pfname+'_t3_resid.eps',xsize=17,ysize=12,/color, Bits_per_pixel=8, /ISOLATIN1, /encap

ploterror,max_sfu/1e6, diff,t3data.t3phierr,$
          psym=8, yr=yr, xr=xr,yst=1,xst=1,position=[0.14,0.13,0.97,0.96], $
 ;         ytit='Closure Phase (!9%!3)', $   ;"
         ytit='Closure Phase (Data - Model) ['+string(176b)+']', $   ;"
;          ytit='Closure Phase ('+string("260B)+')', $   ;"
          xtit="Maximum B/!4k!3 x10!u-6!n within Triangle",/nodata,errthick=1
;          xtit="Maximum !3B/!9l!3 x10!u-6!n within Triangle",/nodata,errthick=1
;          xtit="Maximum !3B/!4k!3 x10!u-6!n within Triangle",/nodata,errthick=1
;xtit="!3B/!9l!3 x10!u-6!n"
oplot,xr,[0,0],linestyle=2

frac = 1.0
usersym, frac*cos(Z), frac*sin(Z), /fill

oploterror,max_sfu/1e6, diff,t3data.t3phierr, psym=8

device,/close

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

set_plot,'x'

multiplot,/reset

;stop,'Type .cont to continue'

end

