pro calc_binary_scale_visph_xy, bin_parameters, u, v, vis, phase, wcent=wcent, wband=wband, viscomplex=viscomplex

; Compute visibility amplitude and phase for a binary star for a given pair 
; of interferometer u,v coordinates.  Input to the routine are (sepRA,sepDEC)
; rather than (sep,pa).  Adjust scaling for incoherent flux from wide
; companion.
;
; CALL TO THE PROGRAM:
; calc_binary_visph_xy, [sepRA_21, sepDEC_21, f1, f2, f3, diam1, diam2],
;                        u, v, vis, phase, wcent=wcent, wband=wband
;
; INPUT PARAMETERS:
; bin_parameters: array containing parameters of binary system
;     - [sepRA_21, sepDEC_21, f1, f2, f3, diam1, diam2]
;     - (sepRA_21, sepDEC_21): separation in (RA, DEC) between components 2 and 1 in mas
;     - Secondary/primary flux ratio as a fraction (0 -> 1)
;     - Tertiary/primary flux ratio as a fraction (0 -> 1)
;     - Component diameters in mas.  Enter 0.0 if diameters are unresolved.
; u_arr, v_arr: arrays specifying u,v coordinates as unitless quantities
;        u = Bx/lambda,  v = By/lambda
; vis:   array to hold visibilities of binary
; phase: array to hold phases of binary
; wcent: central wavelength of the filter or wavelength channel (in meters)
; wband: bandwith of filter or wavelength channel in meters
;
; Note: If wcent and wband are defined, then the bandwidth smearing
; correction will be applied.  Do not define these if you do not want
; to apply the correction.
;
; COMPUTATIONS:
; The complex visibility of a binary is given by:
;
;     Vbin = {V1 + f2*V2*exp[-2*pi*i*(u*sepRA2 + v*sepDEC2)]}/(1 + f2 + f3)
; 
;     V1=2*J1(pi*diam1*B/lambda)/(pi*diam1*B/lambda)
;     V2=2*J1(pi*diam2*B/lambda)/(pi*diam2*B/lambda)
; 
; where V1, V2, and V3 are the uniform disk visibilities of the primary,
; secondary, and tertiary components with angular diameters diam1, diam2, 
; and diam3. f2, f3 are the flux ratios of the secondary and tertiary relative
; to the primary.  B is the baseline length.
;
; If we define f1, f2, f3 to be the fractional contribution of each component,
; where f1 + f2 + f3 = 1.0, then the complex visibility is given by:
;
;     Vbin = {f1*V1 + f2*V2*exp[-2*pi*i*(u*sepRA2 + v*sepDEC2)]}/(f1 + f2 + f3)
;
; Essentially, the incoherent flux from the wide component adjusts the
; scaling of the visibility curves.  For large "close" binary separations, the
; V -> f1/(f1 + f2 + f3) ... the fractional flux contribution of the 
; primary, as expected.
;
; NOTE!!!! The separations must be in mas !!!!
;
; REVISON NOTES:
; 12Oct2012 - Program started by Gail H. Schaefer.
; 30Jun2017 - Shift phase center to the center of light
; 06Sep2018 - Include 180 deg phase shift when passing through
;             visibility null in angular diameter curve.
; 10Jun2019 - Correct vis,phase when passing through null.
;             + Set phase to 180 when star vis is negative.
;             + Then take abs(vis) [this is the part I needed to add].
; 17Sep2019 - Added limb-darkened diameters.  If mu is not given, then
;             set mu1=0, mu2=0 for uniform disk.

;bin_parameters = [sepRA_21, sepDEC_21, f1, f2, f3, diam1, diam2]

sepRA = bin_parameters(0)
sepDEC = bin_parameters(1)

f1 = bin_parameters(2)
f2 = bin_parameters(3)
f3 = bin_parameters(4)

; Are component diameters defined?
npar = n_elements(bin_parameters)
if (npar eq 9) then begin
   diam1 = bin_parameters(5)
   diam2 = bin_parameters(6)
   mu1 = bin_parameters(7)
   mu2 = bin_parameters(8)
endif else begin
   if (npar eq 7) then begin
      diam1 = bin_parameters(5)
      diam2 = bin_parameters(6)
      mu1 = 0.0d
      mu2 = 0.0d
   endif else begin
      diam1 = 0.0d
      diam2 = 0.0d
      mu1 = 0.0d
      mu2 = 0.0d
   endelse
endelse

; Are the filter specifications defined?
; keyword_set returns True (1) if filter is defined, False (0) if undefined

ans_wcent = keyword_set(wcent)
ans_wband = keyword_set(wband)

if (ans_wcent eq 1) and (ans_wband eq 0) then begin
   stop,'Filter bandwidth not specified.'
endif
if (ans_wcent eq 0) and (ans_wband eq 1) then begin
   stop,'Filter cental wavelength not specified.'
endif

; conversion factor for converting separations from mas to radians:
mas = (1.0d/1000.0d)*(1.0d/60.0d)*(1.0d/60.0d)*(!dpi/180.0d)

; If component star diameters are defined, compute uniform disk
; complex visibilities for each component

; Calculate expected single star visibilities

sfu = sqrt(u^2 + v^2)        ; spatial frequency (baseline/wavelength)

ndata = n_elements(sfu)

argJ_1 = !dpi*sfu*diam1*mas
argJ_2 = !dpi*sfu*diam2*mas

J1_1 = beselj(argJ_1,1)
J1_2 = beselj(argJ_2,1)

J3_2_1 = beselj(argJ_1,1.5)
J3_2_2 = beselj(argJ_2,1.5)

; Initialize visibility amplitude arrays (and set to 1.0)
vis_1 = 1.0d + dblarr(n_elements(sfu))
vis_2 = 1.0d + dblarr(n_elements(sfu))

; Star 1

ind = where(argJ_1 gt 0.0,count) ; avoid dividing by zero visibility amplitude 
; Uniform Disk Diameter
; if (count gt 0) then vis_1(ind) = 2.0*J1_1(ind)/(argJ_1(ind))
; Limb Darkened Disk Diameter
if (count gt 0) then vis_1(ind) = ((1.0 - mu1)*J1_1/(argJ_1(ind)) + mu1*sqrt(!dpi/2.0)*J3_2_1/(argJ_1(ind))^(3.0/2.0))/((1.0 - mu1)/2.0 + mu1/3.0)

ind_neg = where(vis_1 lt 0.0, ncount)
phase1 = dblarr(ndata)
if (ncount gt 0) then phase1(ind_neg) = !dpi
vis_1 = abs(vis_1)

factor1 = complex(cos(phase1), sin(phase1))

; Star 2

ind = where(argJ_2 gt 0.0,count) ; avoid dividing by zero visibility amplitude
; Uniform Disk Diameter
;if (count gt 0) then vis_2(ind) = 2.0*J1_2(ind)/(argJ_2(ind))
; Limb Darkened Disk Diameter
if (count gt 0) then vis_2(ind) = ((1.0 - mu2)*J1_2/(argJ_2(ind)) + mu2*sqrt(!dpi/2.0)*J3_2_2/(argJ_2(ind))^(3.0/2.0))/((1.0 - mu2)/2.0 + mu2/3.0)

ind_neg = where(vis_2 lt 0.0, ncount)
phase2 = dblarr(ndata)
if (ncount gt 0) then phase2(ind_neg) = !dpi
vis_2 = abs(vis_2)

factor2 = complex(cos(phase2), sin(phase2))


; Move phase center to center of close pair:
; Use correlated flux to find the photocenter of the pair.

f1c = f1;*mean(vis_1)
f2c = f2;*mean(vis_2)


sepRA_1 = 0.0d - f2c/(f1c+f2c) * sepRA
sepDEC_1 = 0.0d - f2c/(f1c+f2c) * sepDEC

sepRA_2 = (1.0d - f2c/(f1c+f2c)) * sepRA
sepDEC_2 = (1.0d - f2c/(f1c+f2c)) * sepDEC



; Cross-terms

;viscross = complex(cos(-2.0*!dpi*(u*sepRA*mas + v*sepDEC*mas)), $
;                   sin(-2.0*!dpi*(u*sepRA*mas + v*sepDEC*mas)))

viscross1 = complex(cos(-2.0*!dpi*(u*sepRA_1*mas + v*sepDEC_1*mas)), $
                   sin(-2.0*!dpi*(u*sepRA_1*mas + v*sepDEC_1*mas)))

viscross2 = complex(cos(-2.0*!dpi*(u*sepRA_2*mas + v*sepDEC_2*mas)), $
                   sin(-2.0*!dpi*(u*sepRA_2*mas + v*sepDEC_2*mas)))

; Apply bandwidth smearing if requested
if(ans_wcent) then begin

   ; Compute optical path length delays for each component:

   ;opd21 = wcent*(u*sepRA_21*mas + v*sepDEC_21*mas)

   opd1 = wcent*(u*sepRA_1*mas + v*sepDEC_1*mas)
   opd2 = wcent*(u*sepRA_2*mas + v*sepDEC_2*mas)

   ;; Avoid divide by zero:
   ;indz = where(opd21 eq 0.0,countz)
   ;if (countz gt 0) then opd21(indz) = 1d-12
   ;
   ;band2 = sin(!dpi*wband/wcent^2 * opd21) / $
   ;            (!dpi*wband/wcent^2 * opd21)
   ;
   ;visbin = (f1*vis_1 + f2*band2*vis_2*viscross21)/(f1 + f2 + f3)

   ; SET PHASE CENTER TO BE LOCATED AT LIGHT CENTER OF PAIR 12
   ; Avoid divide by zero:
   indz = where(opd1 eq 0.0,countz)
   if (countz gt 0) then opd1(indz) = 1d-12
   indz = where(opd2 eq 0.0,countz)
   if (countz gt 0) then opd2(indz) = 1d-12
   
   band1 = sin(!dpi*wband/wcent^2 * opd1) / $
               (!dpi*wband/wcent^2 * opd1)

   band2 = sin(!dpi*wband/wcent^2 * opd2) / $
               (!dpi*wband/wcent^2 * opd2)

   visbin = (f1*band1*vis_1*factor1*viscross1 + f2*band2*vis_2*factor2*viscross2)/(f1 + f2 + f3)

endif else begin

   ;visbin = (f1*vis_1 + f2*vis_2*viscross21)/(f1 + f2 + f3)
   ;visbin = (f1*vis_1*factor1 + f2*vis_2*factor2*viscross)/(f1 + f2 + f3)

   visbin = (f1*vis_1*factor1*viscross1 + f2*vis_2*factor2*viscross2)/(f1 + f2 + f3)

endelse

re = real_part(visbin)
im = imaginary(visbin)

vis = sqrt(re*re+im*im)
phase=atan(im,re)*180.0/!dpi

viscomplex = visbin

;print,'vis:',vis
;print,'phase:',phase
;print,'V^2:',vis^2

;stop,'Type .cont to continue'

end
