;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; gridsearch_binary_oifits_gui.pro ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; OVERVIEW:
;
; Search through a grid of binary positions to find the minimum chi2 solution. 
; The code uses mpfit to minimize the solution at each point in the grid.  
; The code performs an "adaptive" grid search and adjusts the separation  
; in RA and DEC, as well as the flux ratios at each step in the grid.  
; To perform a fixed grid search (e.g., to map out the chi2 surface at fixed 
; separation intervals) then check the boxes on the GUI to fix the values
; of sepRA and sepDEC.  It is recommended that a best fit should be found 
; using an adaptive grid search first, and then set the best fit as the 
; starting  values to map out the chi2 surface using a fixed grid search.
;
; Running the code will bring up a GUI where the data file can be selected 
; and the initial binary parameters and grid search ranges can be entered.
;
; This version includes an optional scale factor to account for incoherent 
; flux from a wide companion outside of the interferometric field of view.
; If the system is a simple binary with no incoherent flux, then fix
; f3 = 0.
;
; PROCEDURE:
;
; Read in squared visibilities and closure phases from an OIFITS file.
; Extract the (u,v) coordinates from the file to compute calculated values 
; for the model binary based on given separation, position angle, and 
; flux ratio.  Use mpfit program to find best fit binary solution.
;
; By default, this code fits a binary model to both the squared visibilities 
; and closure phases.  To fit only the visibilties or only the closure phases, 
; then select the "Fit V2 only" or "Fit T3 only" buttons on the GUI.
;
; By default, this code includes bandwidth smearing to account for the 
; spread of wavelengths across the filter bandpass or across the wavelength 
; channels.  This is particularly important for wide-band observations.  
; Monochromatic light can be assumed by unclicking the "Include bandwidth 
; smearing" button.
;
; CALL TO THE PROGRAM:
; IDL> gridsearch_binary_oifits_gui
;
; INPUT PARAMETERS:
;   datafile: name of the oifits file
;   parameter array containing parameters of binary system
;     params = [sepRA, sepDEC, f1, f2, f3, diam1, diam2, mu1, mu2]
;     - sepRA, sepDEC: Binary separation in RA and DEC in mas
;     - f1: fractional flux contribution of primary 
;     - f2: fractional flux contribution of secondary 
;     - f3: fractional flux contribution of incoherent light from wide companion
;     - f1 + f2 + f3 = 1.0
;     - diam1, diam2: Component diameters in mas.  If component stars are 
;       unresolved, then fix diameters at 0.0 mas or do not enter the 
;       diam1 and diam2 values (entering only [sepRA, sepDEC, f1, f2, f3]).
;     - mu1, mu2: - Limb-darkening coefficients.  If assuming uniform disk
;       diameters, then fix mu1, mu2 at 0.0 or do not enter mu1, mu2 values.
;
; OUTPUT:
;    Best fitting orbital parameters are printed to the screen.
;    Plots of the data and model are saved in files:
;      filename_t3_bline.eps
;      filename_t3.eps
;      filename_uv_lam.eps
;      filename_uv.eps
;      filename_vis2_big.eps
;      filename_vis2.eps
;    A plot of the chi2 surface is saved as temp.eps
;    (made using auto_chi2_binary_radec.pro)
;    IDL arrays containing results of grid search:
;      ra_array.idl
;      dec_array.idl
;      f1_array.idl
;      f2_array.idl
;      f3_array.idl
;      chi2_array.idl
;      (Plot with plot_chi2_binary_radec.pro or if program is still active,
;       and arrays are still in IDL memory use auto_chi2_binary_radec.pro)
;
; This routine calls the following programs:
;    auto_plot_chi2_binary_radec_scale.pro     (GHS)
;    auto_plot_model_bline_6T.pro              (GHS)
;    calc_binary_vis2cp_seppa_scale_mpfunc.pro (GHS)
;    calc_binary_scale_visph_xy.pro            (GHS)
;    extract_vis2data.pro   (JDM)
;    extract_t3data.pro     (JDM)
;    mpfit
;
; The MPFIT package can be downloaded from the following website:
; https://www.physics.wisc.edu/~craigm/idl/fitting.html
;
; Written by Gail H. Schaefer.  
;
; Based in part on interferometry fitting routines written by John Monnier.
;
; REVISON NOTES:
; 2010Nov26 - GHS: Binary grid search routine started.
; 2018May25 - GHS: Added graphical user interface to input search parameters.
; 2018Aug31 - GHS: Fixed plotting errors when only one data point is plotted.
; 2018Sep05 - GHS: Fixed bounds on sepRA,sepDec when search is not centered 
;                  at (0,0).
; 2018Sep06 - GHS: Include 180 deg phase shift when passing through
;                  visibility null in angular diameter curve.
; 2019Jun10 - Correct vis,phase when passing through null.
;             + Set phase to 180 when star vis is negative.
;             + Then take abs(vis) [this is the part I needed to add].
; 2019Jul31 - Save final results to temp_results in addition to
;             printing to screen.
; 2019Sep17 - Close input parameter file after reading values.
; 2019Sep17 - Added limb-darkened diameters.  If mu is not given, then
;             set mu1=0, mu2=0 for uniform disk diameter.
; 2021Feb25 - For MIRC and MIRC-X data, compute systematic calibration
;             uncertainties in quadrature with formal errors:
;             + 5% uncertainty in V2 calibration
;             + 0.5% uncertainty in wavelength scale for MIRC-X
;             (Anugu et al. 2020)
;             + 0.25% uncertainty in wavelength scale for MIRC
;             (Monnier et al. 2012)
; 2023Jan19 - Update MIRC-X wavelength calibration uncertainties
;             + 5% uncertainty in V2 calibration
;             + 0.06% uncertainty in wavelength scale for MIRC-X
;               (Gardner et al. 2022; Monnier priv. comm. 2022Aug25)
;             + 0.25% uncertainty in wavelength scale for MIRC
;               (Monnier et al. 2012)
; 2023Mar14
;           - Include wavelength calibration uncertainties for MYSTIC:
;             + 5% uncertainty in V2 calibration
;             + 0.07% uncertainty in wavelength scale for MYSTIC
;               (Gardner, private communication ; Monnier priv. comm. 2022Aug25)
;           - The following wavelength scaling factors are automatically 
;             applied for MIRC-X and MYSTIC (sent by JDM on 2022Aug25):
;             + Divide all wavelengths for MIRCX by  1.0054 +/- 0.0006
;               [Gardner et al 2022]
;             + Divide all wavelengths for MYSTIC by 1.0067 +/- 0.0007
;               [Gardner, private communication]
;           - The following wavelength scaling factors are automatically 
;             applied for MIRC (Monnier et al. 2012, ApJL, 761, L3):
;             + Multiple all wavelengths by 0.995
;               [based on iota peg konacki orbit. MIRC4T (before 2011)]
;             + Multiple all wavelengths by 1.004
;               [for 2011-MIRC6T (after 2011)]
;           - Add fix_comb keyword to set beam combiner when INSNAME keyword is
;             incorrect in oifits headers
;             IDL> gridsearch_binary_oifits_gui,fix_comb='MYSTIC'
; 2024May30
;           - Add GUI selections for computing bootstrap uncertainties.
;             + Select measurements at random with repetition.
;             + Randomly vary selected measurements within their uncertainties.
;             + Refit the binary model (use best-fit as starting position).
;             + Repeat for Nsamp number of iterations.
;             + Compute uncertainty ellipse that defines 67% of bootstrap distribution.


PRO gridsearch_binary_oifits_gui_event, event

  ; Set up options for graphical user interface (GUI) to receive input
  ; parameters for the binary grid search.

  COMMON PARAMS, par_ra,par_dec,par_f1,par_f2,par_f3,par_diam1,par_diam2,par_mu1,par_mu2,par_xrange,par_xstep,par_yrange,par_ystep,datafilename,par_nsamp
  COMMON KEYWORDS, fix_RA, fix_DEC, fix_F1, fix_F2, fix_F3, fix_D1, fix_D2, fix_U1, fix_U2, fix_adapt, fix_zcent, check_do_wave, fit_v2, fit_t3,check_do_boot

  WIDGET_CONTROL, event.top, GET_UVALUE=id_state
  WIDGET_CONTROL, event.id, GET_UVALUE=uval

  CASE uval OF

   'O_PARAM' : begin
                 paramfile=dialog_pickfile(title="Select initial parameter file:",$
                   filter="*txt,*dat")
                 sepvalues = strsplit(paramfile,'/',/extract)
                 nz = n_elements(sepvalues)
                 base_paramfile = sepvalues(nz-1)
                 WIDGET_CONTROL, id_state.id_param,  SET_VALUE=base_paramfile
                 
                 close,1
                 openr,1,paramfile
                 tempstring = ''
                 while (not EOF(1)) do begin

                    readf, 1, tempstring

                    ; Check for comment line
                    check = strmid(tempstring,0,1)
                    
                    if (check ne '#') then begin

                       sepvalues = strsplit(tempstring,/extract)
                       nval = n_elements(sepvalues)
                       pval = sepvalues(0)

                       if (pval eq 'Datafile:') then begin
                          datafilename = sepvalues(1)
                          dvalues = strsplit(datafilename,'/',/extract)
                          nz = n_elements(dvalues)
                          base_datafilename = dvalues(nz-1)
                          WIDGET_CONTROL, id_state.id_data, SET_VALUE=base_datafilename
                       endif

                       if (pval eq 'sepRA') then begin
                          par_ra = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_RA = 1
                          endif else fix_RA = 0
                          WIDGET_CONTROL, id_state.id_RA,SET_VALUE=par_ra
                          widget_control, id_state.id_fix_ra, set_button=fix_RA
                       endif

                       if (pval eq 'sepDEC') then begin
                          par_dec = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_DEC = 1
                          endif else fix_DEC = 0
                          WIDGET_CONTROL, id_state.id_DEC,SET_VALUE=par_dec
                          widget_control, id_state.id_fix_dec, set_button=fix_DEC
                       endif

                       if (pval eq 'f1') then begin
                          par_f1 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_f1 = 1
                          endif else fix_f1 = 0
                          WIDGET_CONTROL, id_state.id_f1,SET_VALUE=par_f1
                          widget_control, id_state.id_fix_f1, set_button=fix_f1
                       endif

                       if (pval eq 'f2') then begin
                          par_f2 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_f2 = 1
                          endif else fix_f2 = 0
                          WIDGET_CONTROL, id_state.id_f2,SET_VALUE=par_f2
                          widget_control, id_state.id_fix_f2, set_button=fix_f2
                       endif

                       if (pval eq 'f3') then begin
                          par_f3 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_f3 = 1
                          endif else fix_f3 = 0
                          WIDGET_CONTROL, id_state.id_f3,SET_VALUE=par_f3
                          widget_control, id_state.id_fix_f3, set_button=fix_f3
                       endif

                       if (pval eq 'diam1') then begin
                          par_diam1 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_D1 = 1
                          endif else fix_D1 = 0
                          WIDGET_CONTROL, id_state.id_diam1,SET_VALUE=par_diam1
                          widget_control, id_state.id_fix_d1, set_button=fix_D1
                       endif

                       if (pval eq 'diam2') then begin
                          par_diam2 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_D2 = 1
                          endif else fix_D2 = 0
                          WIDGET_CONTROL, id_state.id_diam2,SET_VALUE=par_diam2
                          widget_control, id_state.id_fix_d2, set_button=fix_D2
                       endif

                       if (pval eq 'mu1') then begin
                          par_mu1 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_U1 = 1
                          endif else fix_U1 = 0
                          WIDGET_CONTROL, id_state.id_mu1,SET_VALUE=par_mu1
                          widget_control, id_state.id_fix_u1, set_button=fix_U1
                       endif

                       if (pval eq 'mu2') then begin
                          par_mu2 = sepvalues(1)
                          if (nval gt 2) then begin
                             if (sepvalues(2) eq 'F') then fix_U2 = 1
                          endif else fix_U2 = 0
                          WIDGET_CONTROL, id_state.id_mu2,SET_VALUE=par_mu2
                          widget_control, id_state.id_fix_U2, set_button=fix_U2
                       endif

                       if (pval eq 'range_sepRA') then begin
                          par_xrange = sepvalues(1)
                          WIDGET_CONTROL, id_state.id_xrange,SET_VALUE=par_xrange
                       endif

                       if (pval eq 'step_sepRA') then begin
                          par_xstep = sepvalues(1)
                          WIDGET_CONTROL, id_state.id_xstep,SET_VALUE=par_xstep
                       endif

                       if (pval eq 'range_sepDEC') then begin
                          par_yrange = sepvalues(1)
                          WIDGET_CONTROL, id_state.id_yrange,SET_VALUE=par_yrange
                       endif

                       if (pval eq 'step_sepDEC') then begin
                          par_ystep = sepvalues(1)
                          WIDGET_CONTROL, id_state.id_ystep,SET_VALUE=par_ystep
                       endif

                       if (pval eq 'fix_adapt') then begin
                          fix_adapt = fix(sepvalues(1))
                          widget_control, id_state.id_fix_adapt, set_button=fix_adapt
                          if (fix_adapt) then begin
                             ; For pseudo-adaptive grid search,
                             ; make sure (sepRA,sepDEC) are free parameters.
                             fix_RA  = 0
                             fix_DEC = 0
                             widget_control, id_state.id_fix_ra, set_button=fix_RA
                             widget_control, id_state.id_fix_dec, set_button=fix_DEC
                          endif
                       endif

                       if (pval eq 'fix_zcent') then begin
                          fix_zcent = fix(sepvalues(1))
                          widget_control, id_state.id_fix_zcent, set_button=fix_zcent
                       endif

                       if (pval eq 'do_wave') then begin
                          check_do_wave = fix(sepvalues(1))
                          widget_control, id_state.id_do_wave, set_button=check_do_wave
                       endif

                       if (pval eq 'do_boot') then begin
                          check_do_boot = fix(sepvalues(1))
                          widget_control, id_state.id_do_boot, set_button=check_do_boot
                       endif

                       if (pval eq 'Nsamp') then begin
                          par_nsamp = sepvalues(1)
                          WIDGET_CONTROL, id_state.id_nsamp,SET_VALUE=par_nsamp
                       endif

                    endif
                 endwhile
                 close,1
               end


   'O_DATA'  : begin
                 datafilename=dialog_pickfile(title="Select data file:",$
                   filter="*.oifits,*.fits")
                 sepvalues = strsplit(datafilename,'/',/extract)
                 nz = n_elements(sepvalues)
                 base_datafilename = sepvalues(nz-1)
                 WIDGET_CONTROL, id_state.id_data, SET_VALUE=base_datafilename
               end

   'ENTER_RA'  : begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_ra
                 end	    
 
   'ENTER_DEC' : begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_dec
                 end	    
 
   'ENTER_F1'  : begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_f1
                 end	    
  
   'ENTER_F2'  : begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_f2
                 end	    

   'ENTER_F3'  : begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_f3
                 end	    

   'ENTER_D1' :  begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_diam1
                 end	    

   'ENTER_D2' :  begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_diam2
                 end	    

   'ENTER_U1' :  begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_mu1
                 end	    

   'ENTER_U2' :  begin
                 WIDGET_CONTROL, event.id,GET_VALUE=par_mu2
                 end	    

   'FIX_RA'    : begin
                 if event.select eq 1 then begin
                    fix_RA = 1
                    ; If fixing (sepRA,sepDEC) then make sure
                    ; FIX_ADAPT is turned off.
                    fix_adapt = 0
                    widget_control, id_state.id_fix_adapt, set_button=fix_adapt
                 endif else fix_RA = 0
                 end

   'FIX_DEC'   : begin
                 if event.select eq 1 then begin
                    fix_DEC = 1
                    ; If fixing (sepRA,sepDEC) then make sure
                    ; FIX_ADAPT is turned off.
                    fix_adapt = 0
                    widget_control, id_state.id_fix_adapt, set_button=fix_adapt
                 endif else fix_DEC = 0
                 end

   'FIX_F1'    : begin
                 if event.select eq 1 then fix_F1 = 1 else fix_F1 = 0
                 end

   'FIX_F2'    : begin
                 if event.select eq 1 then fix_F2 = 1 else fix_F2 = 0
                 end

   'FIX_F3'    : begin
                 if event.select eq 1 then fix_F3 = 1 else fix_F3 = 0
                 end

   'FIX_D1'    : begin
                 if event.select eq 1 then fix_D1 = 1 else fix_D1 = 0
                 end

   'FIX_D2'    : begin
                 if event.select eq 1 then fix_D2 = 1 else fix_D2 = 0
                 end

   'FIX_U1'    : begin
                 if event.select eq 1 then fix_U1 = 1 else fix_U1 = 0
                 end

   'FIX_U2'    : begin
                 if event.select eq 1 then fix_U2 = 1 else fix_U2 = 0
                 end

   'ENTER_XRANGE' : begin
                    WIDGET_CONTROL, event.id,GET_VALUE=par_xrange
                    end	    

   'ENTER_XSTEP'  : begin
                    WIDGET_CONTROL, event.id,GET_VALUE=par_xstep
                    end	    

   'ENTER_YRANGE' : begin
                    WIDGET_CONTROL, event.id,GET_VALUE=par_yrange
                    end	    

   'ENTER_YSTEP'  : begin
                    WIDGET_CONTROL, event.id,GET_VALUE=par_ystep
                    end	    

   'FIX_ADAPT' : begin
                 if event.select eq 1 then begin
                    fix_adapt = 1 
                    ; For pseudo-adaptive grid search,
                    ; make sure (sepRA,sepDEC) are free parameters.
                    fix_RA  = 0
                    fix_DEC = 0
                    widget_control, id_state.id_fix_ra, set_button=fix_RA
                    widget_control, id_state.id_fix_dec, set_button=fix_DEC
                 endif else fix_adapt = 0
                 end

   'FIX_ZCENT' : begin
                 if event.select eq 1 then fix_zcent = 1 else fix_zcent = 0
                 end

   'DO_WAVE'   : begin
                 if event.select eq 1 then check_do_wave = 1 else check_do_wave = 0
                 end

   'FIT_V2_ONLY' : begin
                 if event.select eq 1 then begin
                    fit_v2 = 1
                    ; If fitting only V2 then, make sure fit_t3 is off
                    fit_t3 = 0
                    widget_control, id_state.id_fit_t3, set_button=fit_t3
                 endif 
                 end

   'FIT_T3_ONLY' : begin
                 if event.select eq 1 then begin
                    fit_t3 = 1
                    ; If fitting only V2 then, make sure fit_v2 is off
                    fit_v2 = 0
                    widget_control, id_state.id_fit_v2, set_button=fit_v2
                 endif 
                 end

   'DO_BOOT'   : begin
                 if event.select eq 1 then check_do_boot = 1 else check_do_boot = 0
                 end

   'ENTER_NSAMP'  : begin
                    WIDGET_CONTROL, event.id,GET_VALUE=par_nsamp
                    end	    

   'SAVE_PAR'  : begin
                 close,1
                 openw,1,'temp_param.txt'
                 printf,1,'# OIFITS Data File'
                 printf,1,'Datafile: '+string(datafilename)
                 printf,1,'# Initial Parameters'
                 printf,1,'#   [sepRA, sepDEC, f1, f2, f3, diam1, diam2, mu1, mu2]'
                 printf,1,'#   sepRA, sepDEC: Binary separation in RA and DEC in mas'
                 printf,1,'#   f1: fractional flux contribution of primary '
                 printf,1,'#   f2: fractional flux contribution of secondary'
                 printf,1,'#   f3: fractional flux contribution of incoherent light from a wide component'
                 printf,1,'#       (fix to 0.0 if no incoherent light present)'
                 printf,1,'#   NOTE: f1 + f2 + f3 = 1.0'
                 printf,1,'#   diam1, diam2: Component diameters in mas.  If component stars are '
                 printf,1,'#                 unresolved, then enter diameters as 0.0 mas.'
                 printf,1,'#   mu1, mu2: Component limb-darkening coefficients.'
                 printf,1,'#                 Enter mu1 and mu2 as 0.0 for uniform disk.'
                 printf,1,'#   If you wish to fix any of the parameters, then enter "F" following the value'
                 if (fix_RA) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'sepRA     ',par_RA(0),fix_mark
                 if (fix_DEC) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'sepDEC    ',par_DEC(0),fix_mark
                 if (fix_f1) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'f1        ',par_f1(0),fix_mark
                 if (fix_f2) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'f2        ',par_f2(0),fix_mark
                 if (fix_f3) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'f3        ',par_f3(0),fix_mark
                 if (fix_D1) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'diam1     ',par_diam1(0),fix_mark
                 if (fix_D2) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'diam2     ',par_diam2(0),fix_mark
                  if (fix_U1) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'mu1     ',par_mu1(0),fix_mark
                 if (fix_U2) then fix_mark = ' F' else fix_mark = ''
                 printf,1,'mu2     ',par_mu2(0),fix_mark
                 printf,1,'# Define search ranges:'
                 printf,1,'# The program will search over a two-dimensional grid defined by:'
                 printf,1,'#   (sepRA - sep_range) to (sepRA + sep_range) at step_sepRA steps'
                 printf,1,'#   (sepDEC - sep_range) to (sepDEC + sep_range) at step_sepDEC steps'
                 printf,1,'range_sepRA  ',par_xrange(0)
                 printf,1,'range_sepDEC ',par_yrange(0)
                 printf,1,'step_sepRA   ',par_xstep(0)
                 printf,1,'step_sepDEC  ',par_ystep(0)
                 printf,1,'# Pseudo-adaptive grid search within fixed boxes set by step size at each '
                 printf,1,'# grid point (turn on by setting fix_adapt = 1):'
                 printf,1,'fix_adapt     ',fix(fix_adapt)
                 printf,1,'# Centering grid search at (0,0) using fix_zcent = 1'
                 printf,1,'# otherwise center at (sepRA,sepDEC) using fix_zcent = 0'
                 printf,1,'fix_zcent     ',fix(fix_zcent)
                 printf,1,'# Include bandwidth smearing by setting do_wave = 1:'
                 printf,1,'do_wave       ',fix(check_do_wave)
                 printf,1,'# Compute bootstrap uncertainties by setting do_boot = 1:'
                 printf,1,'do_boot       ',fix(check_do_boot)
                 printf,1,'Nsamp   ',par_nsamp
                 close,1
                 print,'Initial parameters saved to temp_param.txt'
                 end

   'DONE'      : WIDGET_CONTROL, event.TOP, /DESTROY

 ENDCASE

END


PRO gridsearch_binary_oifits_gui, fix_comb=fix_comb

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Set up widget for the graphical user interface ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  COMMON PARAMS, par_ra,par_dec,par_f1,par_f2,par_f3,par_diam1,par_diam2,par_mu1,par_mu2,par_xrange,par_xstep,par_yrange,par_ystep,datafilename,par_nsamp
  COMMON KEYWORDS, fix_RA, fix_DEC, fix_F1, fix_F2, fix_F3, fix_D1, fix_D2, fix_U1, fix_U2, fix_adapt, fix_zcent, check_do_wave,fit_v2,fit_t3,check_do_boot

  ; Initialize datafilename
  datafilename = ''

  ; Initial parameters - default values
  par_ra  = '0.0'
  par_dec = '0.0'
  par_f1  = '0.5'
  par_f2  = '0.5'
  par_f3  = '0.0'
  par_diam1  = '0.0'
  par_diam2  = '0.0'
  par_mu1  = '0.0'
  par_mu2  = '0.0'

  ; Default values for whether to fix parameters (0=free, 1=fixed)
  fix_RA  = 0
  fix_DEC = 0
  fix_F1  = 0
  fix_F2  = 0
  fix_F3  = 1
  fix_D1  = 1
  fix_D2  = 1
  fix_U1  = 1
  fix_U2  = 1

  ; Default search parameters (mas)
  par_xrange = '20.0'
  par_yrange = '20.0'
  par_xstep = '0.5'
  par_ystep = '0.5'

  ; Set fix_adapt=1 for adpative grid search within fixed boxes set by step size
  ; Default is an adaptive grid search with no boundaries (fix_adapt = 0)
  fix_adapt = 0

  ; Center grid search at 0,0 rather than at sepRA,sepDEC
  fix_zcent = 0

  ; Set check_do_wave=1 to include bandwidth smearing.
  ; Set check_do_wave=0 for monochromatic light.
  check_do_wave = 1

  ; Options to fit V2 only or fit T3 only (set to 1 for limited fit)
  ; Default is to fit both V2 and T3 (both set to 0)
  fit_v2 = 0
  fit_t3 = 0

  ; Options for computing bootstrap uncertainties 
  ; Set check_do_boot = 1 to compute bootstrap uncertainties (default is 0 = OFF).
  ; Set default number of bootstrap samples to 1000.
  check_do_boot = 0
  par_nsamp = '1000'
  
  ; Begin creation of widget

  base = WIDGET_BASE(title='Grid Search Parameters',/COLUMN, xsize=740, ysize=900)

  ; Option of opening input parameter file to read in initial values and search ranges

  row1 = widget_base(base, /row)
  button1 = WIDGET_BUTTON(row1, VALUE='Open Param File ', UVALUE='O_PARAM')
  id_param = WIDGET_TEXT(row1, XSIZE=100, /EDITABLE, /ALIGN_RIGHT)

  ; Select oifits data file

  row2 = widget_base(base, /row)
  button2 = WIDGET_BUTTON(row2, VALUE='Select Data File', UVALUE='O_DATA')
  id_data = WIDGET_TEXT(row2, XSIZE=100, /EDITABLE, /ALIGN_RIGHT)

  ; Initial Binary parameters

  row3 = widget_base(base, /row)
  lab3 = widget_label(row3, value = 'Initial Binary Parameters')

  row4 = widget_base(base, /row)
  lab4 = widget_label(row4, value = '    sepRA  (mas): ')
  id_ra = WIDGET_TEXT(row4, value = par_ra, XSIZE=10, /EDITABLE, uvalue='ENTER_RA')
  row4f = widget_base(row4, row=1, frame=1, /Nonexclusive)
  id_fix_ra = widget_button(row4f, value='Fix', uvalue='FIX_RA')
  lab4b = widget_label(row4, value = '   Companion is located at (sepRA, sepDEC) in mas.')
  ; Set default for flag to fix parameters
  widget_control, id_fix_ra, set_button=fix_RA

  row5 = widget_base(base, /row)
  lab5 = widget_label(row5, value = '    sepDEC (mas): ')
  id_dec = WIDGET_TEXT(row5, value = par_dec, XSIZE=10, /EDITABLE, uvalue='ENTER_DEC')
  row5f = widget_base(row5, row=1, frame=1, /Nonexclusive)
  id_fix_dec = widget_button(row5f, value='Fix', uvalue='FIX_DEC') 
  ; Set default for flag to fix parameters
  widget_control, id_fix_dec, set_button=fix_DEC

  row6 = widget_base(base, /row)
  lab6 = widget_label(row6, value = '    f1:           ')
  id_f1 = WIDGET_TEXT(row6, value = par_f1, XSIZE=10, /EDITABLE, uvalue='ENTER_F1')
  row6f = widget_base(row6, row=1, frame=1, /Nonexclusive)
  id_fix_f1 = widget_button(row6f, value='Fix', uvalue='FIX_F1') 
  lab6b = widget_label(row6, value = '   Flux contribution of star 1 (values from 0 to 1).    f1 + f2 + f3 = 1.0')
  ; Set default for flag to fix parameters
  widget_control, id_fix_f1, set_button=fix_F1

  row7 = widget_base(base, /row)
  lab7 = widget_label(row7, value = '    f2:           ')
  id_f2 = WIDGET_TEXT(row7, value = par_f2, XSIZE=10, /EDITABLE, uvalue='ENTER_F2')
  row7f = widget_base(row7, row=1, frame=1, /Nonexclusive)
  id_fix_f2 = widget_button(row7f, value='Fix', uvalue='FIX_F2') 
  lab7b = widget_label(row7, value = '   Flux contribution of star 2 (values from 0 to 1).')
  ; Set default for flag to fix parameters
  widget_control, id_fix_f2, set_button=fix_F2

  row8 = widget_base(base, /row)
  lab8 = widget_label(row8, value = '    f3:           ')
  id_f3 = WIDGET_TEXT(row8, value = par_f3, XSIZE=10, /EDITABLE, uvalue='ENTER_F3')
  row8f = widget_base(row8, row=1, frame=1, /Nonexclusive)
  id_fix_f3 = widget_button(row8f, value='Fix', uvalue='FIX_F3') 
  lab8b = widget_label(row8, value = '   Incoherent flux (values from 0 to 1).  Fix to 0 if no incoherent flux.')
  ; Set default for flag to fix parameters
  widget_control, id_fix_f3, set_button=fix_F3

  row9 = widget_base(base, /row)
  lab9 = widget_label(row9, value = '    Diam1 (mas):  ')
  id_diam1 = WIDGET_TEXT(row9, value = par_diam1, XSIZE=10, /EDITABLE, uvalue='ENTER_D1')
  row9f = widget_base(row9, row=1, frame=1, /Nonexclusive)
  id_fix_d1 = widget_button(row9f, value='Fix', uvalue='FIX_D1') 
  lab9b = widget_label(row9, value = '   Diameters of star 1 and star 2 (in mas)')
  ; Set default for flag to fix parameters
  widget_control, id_fix_d1, set_button=fix_D1

  row10 = widget_base(base, /row)
  lab10 = widget_label(row10, value = '    Diam2 (mas):  ')
  id_diam2 = WIDGET_TEXT(row10, value = par_diam2, XSIZE=10, /EDITABLE, uvalue='ENTER_D2')
  row10f = widget_base(row10, row=1, frame=1, /Nonexclusive)
  id_fix_d2 = widget_button(row10f, value='Fix', uvalue='FIX_D2') 
  lab10b = widget_label(row10, value = '   For unresolved diameters, fix to 0 or fix to estimated size.')
  ; Set default for flag to fix parameters
  widget_control, id_fix_d2, set_button=fix_D2


  row11 = widget_base(base, /row)
  lab11 = widget_label(row11, value = '    mu1:          ')
  id_mu1 = WIDGET_TEXT(row11, value = par_mu1, XSIZE=10, /EDITABLE, uvalue='ENTER_U1')
  row11f = widget_base(row11, row=1, frame=1, /Nonexclusive)
  id_fix_u1 = widget_button(row11f, value='Fix', uvalue='FIX_U1') 
  lab11b = widget_label(row11, value = '   Limb-darkening coefficients.')
  ; Set default for flag to fix parameters
  widget_control, id_fix_u1, set_button=fix_U1

  row12 = widget_base(base, /row)
  lab12 = widget_label(row12, value = '    mu2:          ')
  id_mu2 = WIDGET_TEXT(row12, value = par_mu2, XSIZE=10, /EDITABLE, uvalue='ENTER_U2')
  row12f = widget_base(row12, row=1, frame=1, /Nonexclusive)
  id_fix_u2 = widget_button(row12f, value='Fix', uvalue='FIX_U2') 
  lab12b = widget_label(row12, value = '   For uniform disk diameter, fix to 0.')
  ; Set default for flag to fix parameters
  widget_control, id_fix_u2, set_button=fix_U2


  ; Search ranges and step sizes

  row13 = widget_base(base, /row)
  lab13 = widget_label(row13, value = 'Grid Search Parameters')

  row14 = widget_base(base, /row)
  lab14 = widget_label(row14, value = '    RA range (mas):  ')
  id_xrange = WIDGET_TEXT(row14, value = par_xrange, XSIZE=10, /EDITABLE, uvalue='ENTER_XRANGE')
  lab14 = widget_label(row14, value = '    RA step (mas):  ')
  id_xstep = WIDGET_TEXT(row14, value = par_xstep, XSIZE=10, /EDITABLE, uvalue='ENTER_XSTEP')

  row15 = widget_base(base, /row)
  lab15 = widget_label(row15, value = '    DEC range (mas): ')
  id_yrange = WIDGET_TEXT(row15, value = par_yrange, XSIZE=10, /EDITABLE, uvalue='ENTER_YRANGE')
  lab15 = widget_label(row15, value = '    DEC step (mas): ')
  id_ystep = WIDGET_TEXT(row15, value = par_ystep, XSIZE=10, /EDITABLE, uvalue='ENTER_YSTEP')

  ; Options for adaptive or fixed grid

  row16 = widget_base(base, /row)
  lab16 = widget_label(row16, value = 'For an adaptive grid search, leave sepRA and sepDEC as free parameters.')

  row17 = widget_base(base, /row)
  lab17 = widget_label(row17, value = 'For a grid search at fixed intervals, check the boxes to fix sepRA and sepDEC.')

  ; Set fix_adapt=1 for an adpative grid search within fixed boxes 
  ; set by step size.  Default is an adaptive grid search with no 
  ; boundaries (fix_adapt = 0)
  row18 = widget_base(base, /row)
  row18f = widget_base(row18, row=1, frame=1, /Nonexclusive)
  id_fix_adapt = widget_button(row18f, value='Pseudo-adaptive grid', uvalue='FIX_ADAPT') 
  ; Set default for flag to fix parameters
  widget_control, id_fix_adapt, set_button=fix_adapt
  lab18 = widget_label(row18, value = 'At each grid point, optimize position within a fixed box set by the step size.')

  ; Option for centering grid search at (0,0) using fix_zcent = 1
  ; or (sepRA,sepDEC) using fix_zcent = 0
  row19 = widget_base(base, /row)
  row19f = widget_base(row19, row=1, frame=1, /Nonexclusive)
  id_fix_zcent = widget_button(row19f, value='Grid center at (0,0)', uvalue='FIX_ZCENT') 
  ; Set default for flag to fix center
  widget_control, id_fix_zcent, set_button=fix_zcent
  lab19 = widget_label(row19, value = 'Check to center grid search at (0,0) rather than (sepRA,sepDEC).')

  ;; Option of including wavelength broadening
  ;row20 = widget_base(base, /row)
  ;row20f = widget_base(row20, row=1, frame=1, /Nonexclusive)
  ;id_do_wave = widget_button(row20f, value='Include bandwidth smearing', uvalue='DO_WAVE') 
  ;; Set default for flag to include wavelength smearing
  ;widget_control, id_do_wave, set_button=check_do_wave

  ; Option of including wavelength broadening
  row20 = widget_base(base, /row)
  row20f1 = widget_base(row20, row=1, frame=1, /Nonexclusive)
  id_do_wave = widget_button(row20f1, value='Include bandwidth smearing', uvalue='DO_WAVE') 
  ; Option of computing bootstap uncertainties
  row20f2 = widget_base(row20, row=1, frame=1, /Nonexclusive)
  id_do_boot = widget_button(row20f2, value='Compute bootstrap uncertainties', uvalue='DO_BOOT') 
  lab20 = widget_label(row20, value = '    NSamp: ')
  id_nsamp = WIDGET_TEXT(row20, value = par_nsamp, XSIZE=10, /EDITABLE, uvalue='ENTER_NSAMP')
  ; Set default for flag to include wavelength smearing and bootstrapped uncertainties.
  widget_control, id_do_wave, set_button=check_do_wave
  widget_control, id_do_boot, set_button=check_do_boot

  ; Options of fitting only V2 or only T3 data
  row21 = widget_base(base, /row)
  row21f1 = widget_base(row21, row=1, frame=1, /Nonexclusive)
  id_fit_v2 = widget_button(row21f1, value='Fit V2 only', uvalue='FIT_V2_ONLY') 
  row21f2 = widget_base(row21, row=1, frame=1, /Nonexclusive)
  id_fit_t3 = widget_button(row21f2, value='Fit T3 only', uvalue='FIT_T3_ONLY') 
  lab21 = widget_label(row21, value = 'If unclicked then fit to both V2 and T3 data.')
  ; Set default for flag to fit both V2 and T3
  widget_control, id_fit_v2, set_button=fit_v2
  widget_control, id_fit_t3, set_button=fit_t3

  ; Save initial parameters to a file

  row22 = widget_base(base, /row)
  button22 = WIDGET_BUTTON(row22, VALUE='Save Param File ', UVALUE='SAVE_PAR')
  lab22 = widget_label(row22, value = 'temp_param.txt')

  ; Quit the widget and run the grid search

  row23 = widget_base(base, /row)
  button2 = WIDGET_BUTTON(row23, value='Run Grid Search', UVALUE='DONE',xsize=200,ysize=40)

  ; Collect all of the widget IDs together into a structure

  state = {id_param:id_param, id_data:id_data, id_ra:id_ra, id_dec:id_dec, id_f1:id_f1, id_f2:id_f2, id_f3:id_f3, id_diam1:id_diam1, id_diam2:id_diam2, id_mu1:id_mu1, id_mu2:id_mu2, id_fix_ra:id_fix_ra, id_fix_dec:id_fix_dec, id_fix_f1:id_fix_f1, id_fix_f2:id_fix_f2, id_fix_f3:id_fix_f3, id_fix_d1:id_fix_d1, id_fix_d2:id_fix_d2, id_fix_u1:id_fix_u1, id_fix_u2:id_fix_u2, id_xrange:id_xrange, id_xstep:id_xstep, id_yrange:id_yrange, id_ystep:id_ystep, id_fix_adapt:id_fix_adapt, id_fix_zcent:id_fix_zcent, id_do_wave:id_do_wave, id_do_boot:id_do_boot, id_nsamp:id_nsamp, id_fit_v2:id_fit_v2, id_fit_t3:id_fit_t3}
  
  WIDGET_CONTROL,base,set_uvalue=state

  WIDGET_CONTROL, base, /REALIZE

  XMANAGER, 'gridsearch_binary_oifits_gui', base

  ;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; End creation of widget ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;

  ; Print parameters to screen as a final check:

  print,'Datafile: ',datafilename
  print,'Initial Parameters:'
  if (fix_RA) then fix_mark = ' F' else fix_mark = ''
  print,'sepRA     ',par_RA(0),fix_mark
  if (fix_DEC) then fix_mark = ' F' else fix_mark = ''
  print,'sepDEC    ',par_DEC(0),fix_mark
  if (fix_f1) then fix_mark = ' F' else fix_mark = ''
  print,'f1        ',par_f1(0),fix_mark
  if (fix_f2) then fix_mark = ' F' else fix_mark = ''
  print,'f2        ',par_f2(0),fix_mark
  if (fix_f3) then fix_mark = ' F' else fix_mark = ''
  print,'f3        ',par_f3(0),fix_mark
  if (fix_D1) then fix_mark = ' F' else fix_mark = ''
  print,'diam1     ',par_diam1(0),fix_mark
  if (fix_D2) then fix_mark = ' F' else fix_mark = ''
  print,'diam2     ',par_diam2(0),fix_mark
  if (fix_U1) then fix_mark = ' F' else fix_mark = ''
  print,'mu1     ',par_mu1(0),fix_mark
  if (fix_U2) then fix_mark = ' F' else fix_mark = ''
  print,'mu2     ',par_mu2(0),fix_mark
  print,'Search ranges:'
  print,'RA range     ',par_xrange(0)
  print,'RA step      ',par_xstep(0)
  print,'DEC range    ',par_yrange(0)
  print,'DEC step     ',par_ystep(0)
  if (fix_adapt) then print,'Perform pseudo-adaptive grid search within fixed boxes set by step size at each grid point'
  if (fix_zcent) then print,'Centering grid search at (0,0).' $
                 else print,'Centering grid search at (sepRA,sepDEC)'
  if (check_do_wave) then print,'Include bandwidth smearing.' $
                     else print,'Assume monochromatic light.'

  if (check_do_boot) then begin
     print,'Compute bootstrap uncertainties.'
     print,'Number of bootstrap samples: ',par_nsamp(0)
  endif

  if (fit_v2) then print,'Fitting only V2 data.' $
  else if (fit_t3) then print,'Fitting only T3 data.' $
  else print,'Fitting V2 and T3 data.'

  ans = ''
  print,'Hit enter to continue.'
  read,ans

  ;stop,'Type .cont to continue'

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Begin gridsearch routine ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  parlabel = ['sepRA  ','sepDEC ','f1     ','f2     ','f3     ','diam1  ','diam2  ','mu1    ','mu2    ']

  ; Compute wide band visibilities and closure phases?
  ; Setting to (1) will include bandwidth smearing. [TRUE]
  ; Setting to (0) will NOT include bandwidth smearing. [FALSE]
  do_wave = fix(check_do_wave)

  ; Compute bootstrap uncertainties?
  ; Setting to (1) will compute bootstrap uncertainties. [TRUE]
  ; Setting to (0) will NOT compute bootstrap uncertainties. [FALSE]
  do_boot = fix(check_do_boot)
  nsamp = double(par_nsamp(0))

  ; Set initial parameters based on GUI input:

  params = [double(par_ra(0)),double(par_dec(0)),double(par_f1(0)),double(par_f2(0)),double(par_f3(0)),double(par_diam1(0)),double(par_diam2(0)),double(par_mu1(0)),double(par_mu2(0))]

  ; Save initial parameters

  params_orig = params
  params_best = params
  best_params = params

  ; Define search ranges based on GUI input

  sep_rangeRA  = double(par_xrange(0))
  sep_rangeDEC = double(par_yrange(0))
  step_sepRA   = double(par_xstep(0))
  step_sepDEC  = double(par_ystep(0))

  if (fix_zcent) then begin

     min_sepRA =  params(0) - sep_rangeRA - round(params(0)*10)/10.0
     max_sepRA =  params(0) + sep_rangeRA - round(params(0)*10)/10.0

     min_sepDEC = params(1) - sep_rangeDEC - round(params(1)*10)/10.0 
     max_sepDEC = params(1) + sep_rangeDEC - round(params(1)*10)/10.0 

  endif else begin

     min_sepRA =  params(0)-sep_rangeRA
     max_sepRA =  params(0)+sep_rangeRA

     min_sepDEC =  params(1)-sep_rangeDEC
     max_sepDEC =  params(1)+sep_rangeDEC

  endelse

  num_sepRA = (max_sepRA - min_sepRA)/step_sepRA + 5
  num_sepDEC = (max_sepDEC - min_sepDEC)/step_sepDEC + 5

  ; Arrays to hold results from grid search
  ra_array = dblarr(num_sepRA,num_sepDEC)
  dec_array = dblarr(num_sepRA,num_sepDEC)
  f1_array = dblarr(num_sepRA,num_sepDEC)
  f2_array = dblarr(num_sepRA,num_sepDEC)
  f3_array = dblarr(num_sepRA,num_sepDEC)
  diam1_array = dblarr(num_sepRA,num_sepDEC)
  diam2_array = dblarr(num_sepRA,num_sepDEC)
  mu1_array = dblarr(num_sepRA,num_sepDEC)
  mu2_array = dblarr(num_sepRA,num_sepDEC)
  chi2_array = dblarr(num_sepRA,num_sepDEC)

  ; Set flags if fitting to only V2 or only T3:
  fit_v2_only = fit_v2
  fit_t3_only = fit_t3

  ; Set data file
  datafile = datafilename(0)

  ; Extract visibility and closure phase data:
  extract_vis2data,vis2data,file=datafile
  extract_t3data,t3data,file=datafile

  ; If using a separate file for CP reduction:
  ; (if so, set datafile_cp here)
  ans_cp = keyword_set(datafile_cp)
  if (ans_cp) then begin
     extract_t3data,t3data,file=datafile_cp
  endif

  ; Extract good data

  igood = where(vis2data.flag eq '70')
  vis2data = vis2data[igood]

  igood = where(t3data.flag eq '70')
  t3data = t3data[igood]

  note_select = ''
  ;note_select='Remove short baselines < 107 m'
  ;note_select='Remove short baselines < 70 m'
  ;note_select='Remove short baselines < 35 m'
  ;minbase = 35.0d;108.0d;70.0d;60.0;35.0;60.0;107.0
  ;iselect = where(vis2data.baseline gt minbase)
  ;vis2data = vis2data[iselect]
  ;iselect = where( (t3data.baseline1 gt minbase) and (t3data.baseline2 gt minbase) and (t3data.baseline3 gt minbase))
  ;t3data = t3data[iselect]

  ;note_select='Remove V2 > 2'
  ;maxvis2 = 2.0
  ;iselect = where(vis2data.vis2data lt maxvis2)
  ;vis2data = vis2data[iselect]


  ; Determine combiner for wavelength calibration

  combiner = strcompress(vis2data(0).insname,/remove_all)
  nz = strlen(combiner)
  if (nz gt 5) then begin
     ; Remove '_vN' extensions from MIRCX combiner keyword
     shortcomb = strmid(combiner,0,5)
     if (shortcomb eq 'MIRCX') then combiner = 'MIRCX'
  endif
  if (nz gt 6) then begin
     shortcomb = strmid(combiner,0,6)
     if (shortcomb eq 'MYSTIC') then combiner = 'MYSTIC'
  endif
  if (nz gt 7) then begin
     shortcomb = strmid(combiner,0,7)
     if (shortcomb eq 'PIONIER') then combiner = 'PIONIER'
  endif

  ; Check if more than one combiner is used:
  num_v2 = n_elements(vis2data.vis2data)
  for i=0, num_v2-1 do begin
     checkcomb = strcompress(vis2data(i).insname,/remove_all)
     nz = strlen(checkcomb)
     if (nz gt 5) then begin
        ; Remove '_vN' extensions from MIRCX combiner keyword
        shortcomb = strmid(checkcomb,0,5)
        if (shortcomb eq 'MIRCX') then checkcomb = 'MIRCX'
     endif
     if (nz gt 6) then begin
        shortcomb = strmid(combiner,0,6)
        if (shortcomb eq 'MYSTIC') then checkcomb = 'MYSTIC'
     endif
     if (nz gt 7) then begin
        shortcomb = strmid(combiner,0,7)
        if (shortcomb eq 'PIONIER') then checkcomb = 'PIONIER'
     endif
     if (checkcomb ne combiner) then combiner = 'MIXED'
  endfor

  ; For mystic data, optionally fix combiner to MYSTIC
  ; Early version of the pipeline set INSNAME to MIRCX
  ans_fix = keyword_set(fix_comb)
  if (ans_fix) then combiner = fix_comb

  wavecor = 1.0   ; No wavelength correction
  wavelabel = "No wavelength correction"
  ans_sys = 0
  if (combiner eq 'MIRCX') then begin
     ; sent through email by JDM on 2022Aug25
     wavecor = 1.0d/1.0054d   ; 0.99463
     wavelabel = "MIRCX wavelength calibration by Monnier and Gardner (updated 2022Aug25)"
     wave_precision = 0.0006     
     vis2_precision = 0.05
     sys_label1 = 'MIRC-X: 0.06% uncertainty in wavelength scale'
     sys_label2 = 'MIRC-X: 5% uncertaintiy in visibility calibration'
     ans_sys = 1
     ; Previous values
     ;wave_precision = 0.005
     ;vis2_precision = 0.05
     ;sys_label1 = 'MIRC-X: 0.5% uncertainty in wavelength scale (Anugu et al. 2020)'
     ;sys_label2 = 'MIRC-X: 5% uncertaintiy in visibility calibration'
     ;ans_sys = 1
  endif 
  if (combiner eq 'MYSTIC') then begin
     ; sent through email by JDM on 2022Aug25
     wavecor = 1.0d/1.0067d   ; 0.99334
     wavelabel = "MYSTIC wavelength calibration by Monnier and Gardner (updated 2022Aug25)"
     wave_precision = 0.0007
     vis2_precision = 0.05
     sys_label1 = 'MYSTIC: 0.07% uncertainty in wavelength scale'
     sys_label2 = 'MYSTIC: 5% uncertaintiy in visibility calibration'
     ans_sys = 1
  endif 
  if (combiner eq 'MIRC' or combiner eq 'H_PRISM') then begin
     med_mjd = median(vis2data.mjd,/double)
     if (med_mjd lt 55562.0d) then begin
        wavecor = 0.995 ; based on iota peg konacki orbit. MIRC4T (before 2011).
        wavelabel = "MIRC4T before 2011"
     endif else begin
        wavecor = 1.004   ; increase wavelengths by this amount for 2011-MIRC6T
        wavelabel = "MIRC6T after 2011"
     endelse
     wave_precision = 0.0025
     vis2_precision = 0.05
     sys_label1 = 'MIRC: 0.25% uncertainty in wavelength scale (Monnier et al. 2012)'
     sys_label2 = 'MIRC: 5% uncertaintiy in visibility calibration'
     ans_sys = 1
  endif 
  if (combiner eq 'PIONIER') then begin
     wave_precision = 0.01
     vis2_precision = 0.01
     sys_label1 = 'PIONIER: 1% uncertainty in wavelength scale (Le Bouquin et al. 2011)'
     sys_label2 = 'PIONIER: 1% uncertaintiy in visibility calibration'
     ans_sys = 1
  endif 
  if (combiner eq 'CLIMB') then begin
     wave_precision = 0.01
     vis2_precision = 0.05
     sys_label1 = 'CLIMB: 1% uncertainty in wavelength scale [APPROX]'
     sys_label2 = 'CLIMB: 5% uncertaintiy in visibility calibration [APPROX]'
     ans_sys = 1
  endif 
  ;wavecor = 1.0/((1.0054d + 1.0067d)/2.0)  ; 
  ;wavelabel = "Averaged MIRC-X + MYSTIC wavelength correction"
  ;wave_precision = 0.0007
  ;vis2_precision = 0.05
  ;sys_label1 = 'Averaged MIRC-X + MYSTIC wavelength correction'
  ;sys_label2 = '7% uncertainty in lambda; 5% uncertaintiy in vis'
  ;ans_sys = 1

  ; Apply wavelength corrections to eff_wave, u, v, and sfu arrays:
  ;   u = ucoord/eff_wave,  v = vcoord/eff_wave
  ;   sfu = sqrt(ucoord^2 + vcoord^2)/eff_wave

  vis2data.eff_wave = vis2data.eff_wave * wavecor
  vis2data.u = vis2data.u / wavecor
  vis2data.v = vis2data.v / wavecor
  vis2data.sfu = vis2data.sfu / wavecor

  t3data.eff_wave = t3data.eff_wave * wavecor
  t3data.u1 = t3data.u1 / wavecor
  t3data.v1 = t3data.v1 / wavecor
  t3data.u2 = t3data.u2 / wavecor
  t3data.v2 = t3data.v2 / wavecor
  t3data.u3 = t3data.u3 / wavecor
  t3data.v3 = t3data.v3 / wavecor

  ; Need also to read in telescope pairs/triangles for computing u,v coordinates
  ; for time-averaged visibilities

  oiarray=0
  read_oidata,datafile,oiarray,bb,cc,dd,ee,ff
  ans_oiarray = keyword_set(oiarray)
  if (ans_oiarray) then begin
  
     ; MIRC telescope indexes go from 0-5
     ; MIRC-X telescope indexes go from 1-6
     ix = 1                     ; MIRC-X
     if (combiner eq 'MIRC') then ix = 0 ; MIRC
     base_names = oiarray[vis2data.sta_index - ix].sta_name
     tri_names = oiarray[t3data.sta_index - ix].sta_name
  endif else begin
     basenames = string(vis2data.sta_index)
     triname = string(t3data.sta_index)
  endelse

  ; Remove baselines with a particular telescope:
  ;flagtel = 'E2'
  ;iselect = where((strtrim(base_names(0,*)) ne flagtel) and (strtrim(base_names(1,*)) ne flagtel))
  ;vis2data = vis2data[iselect]
  ;iselect = where((strtrim(tri_names(0,*)) ne flagtel) and (strtrim(tri_names(1,*)) ne flagtel) and (strtrim(tri_names(2,*)) ne flagtel))
  ;t3data = t3data[iselect]

  vis2model=vis2data
  t3model=t3data

  temp=calc_binary_vis2cp_seppa_scale_mpfunc(params,dp, $
            vis2data=vis2data,t3data=t3data,vis2model=vis2model,t3model=t3model,$
            do_wave=do_wave,fit_v2_only=fit_v2_only,fit_t3_only=fit_t3_only)

  ; Set up parameter information for running the mpfit program.

  nparms=n_elements(params)
  parinfo = replicate({value:0.D, fixed:0, limited:[0,0], $
                                  limits:[0.D,0],step:1e-3,tied:''}, nparms)


  ; Tie fluxes together (f1 + f2 + f3 = 1):
  if (fix_F3) then $
     parinfo(3).tied='1.0-P[2]-P[4]' $ ; If fixing f3
  else  $
     parinfo(4).tied='1.0-P[2]-P[3]'   ; If solving for all fluxes

  ; Set to 1 to fix parameter, 0 to free up parameter
  parinfo(0).fixed=fix_RA  ; sepRA
  parinfo(1).fixed=fix_DEC ; sepDEC
  parinfo(2).fixed=fix_F1  ; fractional flux contribution of primary
  parinfo(3).fixed=fix_F2  ; fractional flux contribution of secondary
  parinfo(4).fixed=fix_F3  ; fractional flux contribution of incoherent light from wide tertiary
  parinfo(5).fixed=fix_D1  ; primary angular diameter
  parinfo(6).fixed=fix_D2  ; secondary angular diameter
  parinfo(7).fixed=fix_U1  ; primary angular diameter
  parinfo(8).fixed=fix_U2  ; secondary angular diameter

  ;        0       1   2   3   4   5      6
  ;params=[binsep, pa, f1, f2, f3, diam1, diam2]

  ;parinfo(0).step=0.1
  ;parinfo(1).step=0.1
  ;parinfo(2).step=0.001
  ;parinfo(3).step=0.001
  ;parinfo(4).step=0.001
  ;parinfo(5).step=0.01
  ;parinfo(6).step=0.01

  ; Set bounds on separations:

  sepmax = 60.0

  ramax = max(abs([params(0) - sep_rangeRA,params(0) + sep_rangeRA]))
  decmax = max(abs([params(1) - sep_rangeDEC,params(1) + sep_rangeDEC]))

  if (sepmax lt ramax)  then sepmax = 1.1*ramax
  if (sepmax lt decmax) then sepmax = 1.1*decmax

  parinfo(0).limited=[1,1]
  parinfo(0).limits=[-sepmax,sepmax]
  parinfo(1).limited=[1,1]
  parinfo(1).limits=[-sepmax,sepmax]
  parinfo(2).limited=[1,1]
  parinfo(2).limits=[0.0,1.0]
  parinfo(3).limited=[1,1]
  parinfo(3).limits=[0.0,1.0]
  parinfo(4).limited=[1,1]
  parinfo(4).limits=[-0.1,1.0]
  parinfo(5).limited=[1,1]
  ;parinfo(5).limits=[0.00,2.0]
  parinfo(5).limits=[0.00,50.0]
  parinfo(6).limited=[1,1]
  ;parinfo(6).limits=[0.00,2.0]
  parinfo(6).limits=[0.00,50.0]
  parinfo(7).limited=[1,1]
  parinfo(7).limits=[0.00,1.0]
  parinfo(8).limited=[1,1]
  parinfo(8).limits=[0.00,1.0]

  ; Note: ftol, gtol, xtol tell mpfit when to terminiate (default 1d-10)
  ;   FTOL - a nonnegative input variable. Termination occurs when both
  ;          the actual and predicted relative reductions in the sum of
  ;          squares are at most FTOL (and STATUS is accordingly set to
  ;          1 or 3).  Therefore, FTOL measures the relative error
  ;          desired in the sum of squares.  Default: 1D-10
  ;   GTOL - cosine of the angle between fvec and any column of the jacobian.
  ;          Measures orthogonality.
  ;   XTOL - relative error between two consecutive iterates.
  ;          Measures the relative error desired in the approximate solution.

  fa={vis2data:vis2data,t3data:t3data,vis2model:vis2model,t3model:t3model,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

  ; Compute initial best guess:

  perror = dblarr(nparms)
  covar = dblarr(nparms,nparms)

  best_params = mpfit('calc_binary_vis2cp_seppa_scale_mpfunc', params, $
                      functargs=fa,bestnorm=bestnorm,parinfo=parinfo, $
                      perror=perror,covar=covar,status=status,xtol=1d-7,/quiet)

  resid=calc_binary_vis2cp_seppa_scale_mpfunc(best_params,dp, $
            vis2data=vis2data,t3data=t3data,vis2model=vis2model,t3model=t3model,$
            do_wave=do_wave,fit_v2_only=fit_v2_only,fit_t3_only=fit_t3_only)

  ; Compute chi2 based on residuals (depends on whether fitting all data,
  ; V2 only, or T3 only

  chi2_all = total(resid^2)
  chi2_best = chi2_all
  params_best = best_params
  perror_best = perror
  covar_best = covar

  ; Begin grid search

  isep = 0
  for sepRA=min_sepRA, max_sepRA, step_sepRA do begin
     jsep = 0
     for sepDEC=min_sepDEC, max_sepDEC, step_sepDEC do begin

        ;print,'SepRA = ',sepRA,'   sepDEC = ',sepDEC

        params = params_orig
        params(0) = sepRA
        params(1) = sepDEC

        ; Fix search ranges to be within sepRA +/- stepRA and sepDEC +/- stepDEC
        ; if fix_adapt is set to 1:
        if (fix_adapt) then begin
           parinfo(0).step=step_sepRA/100.0
           parinfo(1).step=step_sepDEC/100.0
           parinfo(0).limits=[sepRA-step_sepRA,sepRA+step_sepRA] 
           parinfo(1).limits=[sepDEC-step_sepDEC,sepDEC+step_sepDEC] 
        endif


        ;stop,'Type .cont to continue'

        fa={vis2data:vis2data,t3data:t3data,vis2model:vis2model,t3model:t3model,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

        best_params = mpfit('calc_binary_vis2cp_seppa_scale_mpfunc', params, $
                            functargs=fa,bestnorm=bestnorm,parinfo=parinfo, $
                            perror=perror,covar=covar,status=status,xtol=1d-7,/quiet)


        resid=calc_binary_vis2cp_seppa_scale_mpfunc(best_params,dp, $
           vis2data=vis2data,t3data=t3data,vis2model=vis2model,t3model=t3model,$
           do_wave=do_wave,fit_v2_only=fit_v2_only,fit_t3_only=fit_t3_only)

        ; Compute chi2:
        ;
        ; Note: because we change around within calc_binary_vis2cp_seppa_mpfunc
        ; whether we are fitting for vis2, cp, or both, let's compute best
        ; chi2 based on residuals computed by calc_binary_vis2cp_seppa_mpfunc:

        chi2_all = total(resid^2)

        if (chi2_all le chi2_best) then begin
           chi2_best = chi2_all
           params_best = best_params
           perror_best = perror
           covar_best = covar
        endif

        ; Save variables for plotting chi2 surfaces:

        ra_array(isep,jsep) = best_params(0)
        dec_array(isep,jsep) = best_params(1)
        f1_array(isep,jsep) = best_params(2)
        f2_array(isep,jsep) = best_params(3)
        f3_array(isep,jsep) = best_params(4)
        diam1_array(isep,jsep) = best_params(5)
        diam2_array(isep,jsep) = best_params(6)
        mu1_array(isep,jsep) = best_params(7)
        mu2_array(isep,jsep) = best_params(8)
        chi2_array(isep,jsep) = chi2_all

        ;print,'---------------------------------------------------------------'
        ;print,'sepRA  sepDEC   RA   DEC'
        ;print,best_params(0),best_params(1),sepRA,sepDEC
        print,format='("SepRA = ",f10.5,"  sepDEC = ",f10.5,"  BestRA = ",f10.5,"  BestDEC = ",f10.5,"  chi2 = ",f12.3)',sepRA,sepDEC,best_params(0),best_params(1),chi2_all
        ;print,'---------------------------------------------------------------'
    
        jsep = jsep + 1
     endfor
     isep = isep + 1
  endfor

  ra_array = ra_array(0:isep-1,0:jsep-1)
  dec_array = dec_array(0:isep-1,0:jsep-1)
  f1_array = f1_array(0:isep-1,0:jsep-1)
  f2_array = f2_array(0:isep-1,0:jsep-1)
  f3_array = f3_array(0:isep-1,0:jsep-1)
  diam1_array = diam1_array(0:isep-1,0:jsep-1)
  diam2_array = diam2_array(0:isep-1,0:jsep-1)
  mu1_array = mu1_array(0:isep-1,0:jsep-1)
  mu2_array = mu2_array(0:isep-1,0:jsep-1)
  chi2_array = chi2_array(0:isep-1,0:jsep-1)

  params = params_best

  resid=calc_binary_vis2cp_seppa_scale_mpfunc(params_best,dp, $
           vis2data=vis2data,t3data=t3data,vis2model=vis2model,t3model=t3model,$
           do_wave=do_wave,fit_v2_only=fit_v2_only,fit_t3_only=fit_t3_only)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Compute chi2 for V2 and T3 data separately ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  chi2_all = total([ (vis2data.vis2data - vis2model.vis2data)^2/(vis2data.vis2err)^2,$
                     (angle_diff(t3data.t3phi,t3model.t3phi))^2/(t3data.t3phierr)^2])

  chi2_v2 = total([ (vis2data.vis2data - vis2model.vis2data)^2/(vis2data.vis2err)^2])

  chi2_t3 = total([(angle_diff(t3data.t3phi,t3model.t3phi))^2/(t3data.t3phierr)^2])


  indfreepar = where(parinfo.fixed ne 1.0, nfreepar)

  num_v2 = n_elements(vis2data.vis2data) 
  num_t3 = n_elements(t3data.t3phi)
  num_all = num_v2 + num_t3

  redchi_all = chi2_all/(num_all - nfreepar)
  redchi_v2 = chi2_v2/(num_v2 - nfreepar)
  redchi_t3 = chi2_t3/(num_t3 - nfreepar)

  ; Unscaled Residuals
  resid_V2 = vis2data.vis2data - vis2model.vis2data
  resid_T3 = angle_diff(t3data.t3phi,t3model.t3phi)





;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Generate bootstrap errors 
;;;;;;;;;;;;;;;;;;;;;;;;;;;

if (do_boot) then begin

   print,'STARTING BOOTSTRAP ITERATIONS'

   ; Define starting seed.
   ; Use current time as long variable as mmss (6060 possibilities)
   time_str = systime(0)
   sepvalues = strsplit(time_str,/extract)
   time_ext = sepvalues(3)
   sepvalues = strsplit(time_ext,':',/extract)
   seed = long(sepvalues(1)+sepvalues(2))

   ra_boot = dblarr(nsamp)
   dec_boot = dblarr(nsamp)
   f1_boot = dblarr(nsamp)
   f2_boot = dblarr(nsamp)
   f3_boot = dblarr(nsamp)
   diam1_boot = dblarr(nsamp)
   diam2_boot = dblarr(nsamp)
   mu1_boot = dblarr(nsamp)
   mu2_boot = dblarr(nsamp)
   
   ; Begin for loop to compute bootstrap errors
   for ns=0, nsamp-1 do begin

      print,'Bootstrap Iteration: ',ns

      vis2boot = vis2data
      t3boot = t3data
   
      ; Re-sample measurements with replacement
      ind_v2 = fix(num_v2*randomu(seed,num_v2,/uniform))        
      vis2boot.u = vis2data(ind_v2).u
      vis2boot.v = vis2data(ind_v2).v
      vis2boot.vis2data = vis2data(ind_v2).vis2data
      vis2boot.vis2err = vis2data(ind_v2).vis2err

      ; Re-sample measurements with replacement
      ind_t3 = fix(num_t3*randomu(seed,num_t3,/uniform))        
      t3boot.u1 = t3data(ind_t3).u1
      t3boot.v1 = t3data(ind_t3).v1
      t3boot.u2 = t3data(ind_t3).u2
      t3boot.v2 = t3data(ind_t3).v2
      t3boot.u3 = t3data(ind_t3).u3
      t3boot.v3 = t3data(ind_t3).v3
      t3boot.t3phi = t3data(ind_t3).t3phi
      t3boot.t3phierr = t3data(ind_t3).t3phierr
      t3boot.t3amp = t3data(ind_t3).t3amp
      t3boot.t3amperr = t3data(ind_t3).t3amperr

      ; Compute new array of resampled visibilities based on 
      ; randomly distributed errors
      vis2boot.vis2data = vis2boot.vis2data + vis2boot.vis2err*randomn(seed,num_v2,/normal) 
      t3boot.t3phi = t3boot.t3phi + t3boot.t3phierr*randomn(seed,num_t3,/normal) 

      params = params_best

      ; Vary starting values of f1 and f2
      fmin = 0.1
      fmax = 0.7
      frange = fmax - fmin
      f1 = fmin + frange*randomn(seed,/uniform) 
      if (1 - f1 lt fmax) then fmax = 1.0 - f1
      frange = fmax - fmin
      f2 = fmin + frange*randomn(seed,/uniform) 
      f3 = 1.0 - f1 - f2
      params(2) = f1
      params(3) = f2
      params(4) = f3
      
      fa_boot={vis2data:vis2boot,t3data:t3boot,vis2model:vis2model,t3model:t3model,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

      boot_params = mpfit('calc_binary_vis2cp_seppa_scale_mpfunc', params, $
                            functargs=fa_boot,bestnorm=bestnorm,parinfo=parinfo, $
                            perror=perror,covar=covar,status=status,xtol=1d-7,/quiet)

        
      ra_boot(ns) = boot_params(0)
      dec_boot(ns) = boot_params(1)
      f1_boot(ns) = boot_params(2)
      f2_boot(ns) = boot_params(3)
      f3_boot(ns) = boot_params(4)
      diam1_boot(ns) = boot_params(5)
      diam2_boot(ns) = boot_params(6)
      mu1_boot(ns) = boot_params(7)
      mu2_boot(ns) = boot_params(8)

   endfor

   plot,ra_boot,dec_boot,psym=7,/isotropic, $
     xrange=[max(ra_boot),min(ra_boot)],yrange=[min(dec_boot),max(dec_boot)], $
     xtitle='sepRA (mas)',ytitle='sepDEC (mas)'

   ans_select = 'n'
   print,'Plotting the bootstrap distribution.'
   print,'If there are two groups of solutions mirrored about the origin, do you want to select only one group? Or do you want to trim outliers? (y/n)'
   read,ans_select
   while (ans_select eq 'y') do begin

      plot,ra_boot,dec_boot,psym=7,/isotropic, $
           xrange=[max(ra_boot),min(ra_boot)],yrange=[min(dec_boot),max(dec_boot)], $
           xtitle='sepRA (mas)',ytitle='sepDEC (mas)'

      print,'---------------------------------------------'
      print,'You will click on two locations to define the '
      print,'upper-left and lower-right conrners of a box'
      print,'that defines the region around the selected '
      print,'portion of the bootstrap distribution.'
      print,'STEP 1: Click on upper-left corner:'
      cursor,x1,y1,4
      print,'STEP 2: Click on lower-right corner:'
      cursor,x2,y2,4

      oplot,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1]

     
      print, 'Do you need to redefine the selection region (y/n)?'
      read,ans_select
      
   endwhile

   
   ind = where((ra_boot le x1) and (ra_boot ge x2) and (dec_boot le y1) and (dec_boot ge y2))
   
   nsamp_orig = nsamp
   nsamp = n_elements(ind)
   
   ra_boot = ra_boot(ind)
   dec_boot = dec_boot(ind)
   f1_boot = f1_boot(ind)
   f2_boot = f2_boot(ind)
   f3_boot = f3_boot(ind)
   diam1_boot = diam1_boot(ind)
   diam2_boot = diam2_boot(ind)
   mu1_boot = mu1_boot(ind)
   mu2_boot = mu2_boot(ind)

endif

params = params_best

  ;;;;;;;;;;;;;;;; 
  ; Plot results ;
  ;;;;;;;;;;;;;;;;

  auto_plot_model_bline_6T, vis2data, t3data, oiarray, vis2model, t3model, datafile

  ; Decide whether to plot contours on chi2maps.
  ; Typically, let's do this if doing a fixed step grid search.
  ; That way it's more likely that the confidence intervals
  ; will be well defined.
  if (fix_RA) and (fix_DEC) then plot_cont=1 else plot_cont=0

  auto_plot_chi2_binary_radec_scale,ra_array,dec_array,chi2_array,num_all,/rainbow,xrange=[min_sepRA,max_sepRA],yrange=[min_sepDEC,max_sepDEC],plot_cont=plot_cont

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Compute Derived Parameters ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  ; Calculate sep and PA for close pair
  ; sepRA  = sep*sin(pa) and sepDEC = sep*cos(pa)

  ; First scale covar_best by reduced chi2 
  ; Note: I used to do this as the last step after computing uncertainty ellipse
  ; (see commented line below)

  covar_best = covar_best*redchi_all

  sepRA = params_best(0)
  sepDEC = params_best(1)

  dsepRA  = sqrt(covar_best(0,0)) ; perror_best(0)*sqrt(redchi_all)
  dsepDEC = sqrt(covar_best(1,1)) ; perror_best(1)*sqrt(redchi_all)

  sep = sqrt(sepRA^2 + sepDEC^2)
  ; Get quadrant right by putting RA,DEC separately into atan:
  PA = atan(sepRA,sepDEC)*180/!dpi

  ; propagate errors to sep,PA:

  dsep = sqrt((sepRA*dsepRA)^2 + (sepDEC*dsepDEC)^2)/sep

  dPA  = sqrt((dsepDEC*sepRA/sepDEC^2)^2 + (dsepRA/sepDEC)^2)*(cos(PA*!dpi/180.0))^2  *180.0/!dpi

  if (PA lt 0) then PA = PA+360.0

  fratio = params_best(3)/params_best(2)
  dfratio = fratio*sqrt( (perror_best(2)*sqrt(redchi_all)/params_best(2))^2 +(perror_best(3)*sqrt(redchi_all)/params_best(3))^2 )

  ; Compute major, minor axes and orientation of error ellipse (sepRA,sepDEC):
  ; lengths of the ellpse axes are given by the square root of the
  ; eigenvalues of the covariance matrix:
  lam1 = 0.5* (covar_best(0,0) + covar_best(1,1) + sqrt((covar_best(0,0) - covar_best(1,1))^2 + 4*covar_best(0,1)*covar_best(1,0)))
  lam2 = 0.5* (covar_best(0,0) + covar_best(1,1) - sqrt((covar_best(0,0) - covar_best(1,1))^2 + 4*covar_best(0,1)*covar_best(1,0)))

  ; Position angle between y-axis (north) and major axis:
  theta = acos((covar_best(1,0))/sqrt((lam1 - covar_best(1,1))^2 + covar_best(1,0)^2) ) * 180.0/!dpi

  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
  ; Compute systematic uncertainties ;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

  if (ans_sys) then begin

     ; Apply wavelength corrections to eff_wave, u, v, and sfu arrays:
     ;   u = ucoord/eff_wave,  v = vcoord/eff_wave
     ;   sfu = sqrt(ucoord^2 + vcoord^2)/eff_wave

     vis2correct = vis2data
     t3correct = t3data
 
     wave_sys = [wavecor - wave_precision, wavecor + wave_precision]

     sep_sys = dblarr(2)
     f1_sys = dblarr(2)
     f2_sys = dblarr(2)
     f3_sys = dblarr(2)
     diam1_sys = dblarr(2)
     diam2_sys = dblarr(2)

     ; Vary wavelength scale

     for i=0, 1 do begin

        ; NOTE - need to unscale by diving/multipying by wavecor.
        ; Then apply new scaling by multiplying/dividing by wave_sys.
        ; Otherwise we are applying wavecor twice -
        ; vis2data and t3 data already have wavecor applied.
        
        vis2correct.eff_wave = vis2data.eff_wave * wave_sys(i) / wavecor
        vis2correct.u = vis2data.u * wavecor / wave_sys(i)
        vis2correct.v = vis2data.v * wavecor / wave_sys(i)
        vis2correct.sfu = vis2data.sfu * wavecor / wave_sys(i)

        t3correct.eff_wave = t3data.eff_wave * wave_sys(i) / wavecor
        t3correct.u1 = t3data.u1 * wavecor / wave_sys(i)
        t3correct.v1 = t3data.v1 * wavecor / wave_sys(i)
        t3correct.u2 = t3data.u2 * wavecor / wave_sys(i)
        t3correct.v2 = t3data.v2 * wavecor / wave_sys(i)
        t3correct.u3 = t3data.u3 * wavecor / wave_sys(i)
        t3correct.v3 = t3data.v3 * wavecor / wave_sys(i)

        vis2cormod = vis2correct
        t3cormod = t3correct

        params = params_best

        fa={vis2data:vis2correct,t3data:t3correct,vis2model:vis2cormod,t3model:t3cormod,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

        ;fa={vis2data:vis2data,t3data:t3data,vis2model:vis2model,t3model:t3model,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

        params_sys = mpfit('calc_binary_vis2cp_seppa_scale_mpfunc', params, $
                            functargs=fa,bestnorm=bestnorm,parinfo=parinfo, $
                            status=status,xtol=1d-7,/quiet)

        sep_sys(i) = sqrt(params_sys(0)^2 + params_sys(1)^2)
        f1_sys(i) = params_sys(2)
        f2_sys(i) = params_sys(3)
        f3_sys(i) = params_sys(4)
        diam1_sys(i) = params_sys(5)
        diam2_sys(i) = params_sys(6)

     endfor

     sig_sep_wave = abs(sep_sys(1) - sep_sys(0))/2.0
     sig_f1_wave = abs(f1_sys(1) - f1_sys(0))/2.0
     sig_f2_wave = abs(f2_sys(1) - f2_sys(0))/2.0
     sig_f3_wave = abs(f3_sys(1) - f3_sys(0))/2.0
     sig_diam1_wave = abs(diam1_sys(1) - diam1_sys(0))/2.0
     sig_diam2_wave = abs(diam2_sys(1) - diam2_sys(0))/2.0

     ;stop,'Check wavelegth correction'

     ; Vary visibility precision
    
     vis2correct = vis2data
     t3correct = t3data
 
     vis2_sys = [-vis2_precision, vis2_precision]

     for i=0, 1 do begin

        vis2correct.vis2data = vis2data.vis2data + vis2_sys(i)*vis2data.vis2data

        vis2cormod = vis2correct
        t3cormod = t3correct

        params = params_best

        fa={vis2data:vis2correct,t3data:t3correct,vis2model:vis2cormod,t3model:t3cormod,do_wave:do_wave,fit_v2_only:fit_v2_only,fit_t3_only:fit_t3_only}

        params_sys = mpfit('calc_binary_vis2cp_seppa_scale_mpfunc', params, $
                            functargs=fa,bestnorm=bestnorm,parinfo=parinfo, $
                            status=status,xtol=1d-7,/quiet)

        sep_sys(i) = sqrt(params_sys(0)^2 + params_sys(1)^2)
        f1_sys(i) = params_sys(2)
        f2_sys(i) = params_sys(3)
        f3_sys(i) = params_sys(4)
        diam1_sys(i) = params_sys(5)
        diam2_sys(i) = params_sys(6)

     endfor

     sig_sep_vis = abs(sep_sys(1) - sep_sys(0))/2.0
     sig_f1_vis = abs(f1_sys(1) - f1_sys(0))/2.0
     sig_f2_vis = abs(f2_sys(1) - f2_sys(0))/2.0
     sig_f3_vis = abs(f3_sys(1) - f3_sys(0))/2.0
     sig_diam1_vis = abs(diam1_sys(1) - diam1_sys(0))/2.0
     sig_diam2_vis = abs(diam2_sys(1) - diam2_sys(0))/2.0

     ;stop,'Check visibility correction'

     sig_major_sys = sqrt(lam1 + sig_sep_wave^2 + sig_sep_vis^2)
     sig_minor_sys = sqrt(lam2) * sig_major_sys/ sqrt(lam1)
     sig_PA_sys = theta
     sig_f1 = perror_best(2)*sqrt(redchi_all)
     sig_f1_sys = sqrt(sig_f1^2 + sig_f1_wave^2 + sig_f1_vis^2)
     sig_f2 = perror_best(3)*sqrt(redchi_all)
     sig_f2_sys = sqrt(sig_f2^2 + sig_f2_wave^2 + sig_f2_vis^2)
     sig_f3 = perror_best(4)*sqrt(redchi_all)
     sig_f3_sys = sqrt(sig_f3^2 + sig_f3_wave^2 + sig_f3_vis^2)
     sig_diam1 = perror_best(5)*sqrt(redchi_all)
     sig_diam1_sys = sqrt(sig_diam1^2 + sig_diam1_wave^2 + sig_diam1_vis^2)
     sig_diam2 = perror_best(6)*sqrt(redchi_all)
     sig_diam2_sys = sqrt(sig_diam2^2 + sig_diam2_wave^2 + sig_diam2_vis^2)

     fratio = params_best(3)/params_best(2)
     sig_fratio_sys = fratio*sqrt((sig_f1_sys/params_best(2))^2 +(sig_f3_sys/params_best(3))^2 )

  endif else begin

     ; If no systematic uncertainties are calculated, then set calues equal to formal errors
                            
     sig_major_sys = sqrt(lam1)
     sig_minor_sys = sqrt(lam2)
     sig_PA_sys = theta
     sig_f1 = perror_best(2)*sqrt(redchi_all)
     sig_f1_sys = sig_f1
     sig_f2 = perror_best(3)*sqrt(redchi_all)
     sig_f2_sys = sig_f2
     sig_f3 = perror_best(4)*sqrt(redchi_all)
     sig_f3_sys = sig_f3
     sig_diam1 = perror_best(5)*sqrt(redchi_all)
     sig_diam1_sys = sig_diam1
     sig_diam2 = perror_best(6)*sqrt(redchi_all)
     sig_diam2_sys = sig_diam2

     fratio = params_best(3)/params_best(2)
     sig_fratio_sys = fratio*sqrt((sig_f1_sys/params_best(2))^2 +(sig_f3_sys/params_best(3))^2 )

     
  endelse

  ;;;;;;;;;;;;;;;;;;;;;;
  ; Summary of Results ;
  ;;;;;;;;;;;;;;;;;;;;;;

  print,'---------------------------------------------------------------'
  if (ans_cp) then begin
     print,'V2 Data file: ', datafile
     print,'T3 Data file: ', datafile_cp
  endif else begin
     print,'Data file: ', datafile
  endelse
  print,'Best fit total(resid^2) from mpfit: ',total(resid^2)
  print,'All chi2:',chi2_all,num_all,redchi_all
  print,'V2 chi2:',chi2_v2,num_v2,redchi_v2
  print,'T3 chi2:',chi2_t3,num_t3,redchi_t3
  print,'Number of free parameters:',nfreepar

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;; From mpfit.pro ;;;;;;;;;;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;   PERROR - The formal 1-sigma errors in each parameter, computed
  ;            from the covariance matrix.  If a parameter is held
  ;            fixed, or if it touches a boundary, then the error is
  ;            reported as zero.
  ;
  ;            If the fit is unweighted (i.e. no errors were given, or
  ;            the weights were uniformly set to unity), then PERROR
  ;            will probably not represent the true parameter
  ;            uncertainties.  
  ;
  ;            *If* you can assume that the true reduced chi-squared
  ;            value is unity -- meaning that the fit is implicitly
  ;            assumed to be of good quality -- then the estimated
  ;            parameter uncertainties can be computed by scaling PERROR
  ;            by the measured chi-squared value.
  ;
  ;              DOF     = N_ELEMENTS(X) - N_ELEMENTS(PARMS) ; deg of freedom
  ;              PCERROR = PERROR * SQRT(BESTNORM / DOF)   ; scaled uncertainties

  print,'Parameter Error'
  ;for i=0, nparms-1 do print,parlabel(i),params_best(i),perror_best(i)*sqrt(redchi_all),sqrt(covar_best(i,i))
  for i=0, nparms-1 do print,parlabel(i),params_best(i),perror_best(i)*sqrt(redchi_all)

  print,'Close pair separation:  ',sep,' +/- ',dsep
  print,'Close pair position angle: ',PA,' +/- ',dPA
  print,'Close pair fratio (f2/f1): ',fratio,' +/- ',dfratio

  print,'Flux ratio of wide component relative to primary: ',params_best(4)/params_best(2)

  ;print,'Covariance matrix for sepRA,sepDEC:'
  ;print,covar_best(0:1,0:1)
  ;print,'Error ellipse - major, minor, PA:',sqrt(lam1)*sqrt(redchi_all),sqrt(lam2)*sqrt(redchi_all),theta
  ; Note: Now I scale covar_best before adding systematics in quadrature (see equation above)
  print,'Error ellipse - major, minor, PA:',sqrt(lam1),sqrt(lam2),theta

  if (ans_sys) then begin
  print,'---------------------------------------------------------------'
  print,'Add systematic calibration uncertainties in quadrature with formal errors:'
  print,'- ',sys_label1
  print,'- ',sys_label2
  print,'Total error ellipse - major, minor, PA: ',sig_major_sys,sig_minor_sys,sig_pa_sys
  print,'Total errors in f1, f2, f3: ',sig_f1_sys,sig_f2_sys,sig_f3_sys
  print,'Total errors in fratio: ',sig_fratio_sys
  print,'Total errors in diam1, diam2: ',sig_diam1_sys, sig_diam2_sys
  print,'---------------------------------------------------------------'
  endif

  print,'Wavelength correction factor: ',string(format='(f7.4)',wavecor),'    -- ',wavelabel

  if (do_wave) then print,'Fit includes bandwidth smearing.' $
               else print,'Fit assumes monochromatic light.'

  if (fit_v2) then print,'Fitting only V2 data.' $
  else if (fit_t3) then print,'Fitting only T3 data.' $
  else print,'Fitting V2 and T3 data.'

  print,'---------------------------------------------------------------'


if (do_boot) then begin

   sep_boot = sqrt(ra_boot^2 + dec_boot^2)
   ; Get quadrant right by putting RA,DEC separately into atan:
   PA_boot = atan(ra_boot,dec_boot)*180/!dpi

   print,'---------------------------------------------------------------'
   print,'Bootstrap Uncertainties (min max mean median stddev):'
   print,'sepRA: ',min(ra_boot),max(ra_boot),mean(ra_boot),median(ra_boot),stdev(ra_boot)
   print,'sepDEC:',min(dec_boot),max(dec_boot),mean(dec_boot),median(dec_boot),stdev(dec_boot)
   print,'sep:   ',min(sep_boot),max(sep_boot),mean(sep_boot),median(sep_boot),stdev(sep_boot)
   print,'PA:    ',min(PA_boot),max(PA_boot),mean(PA_boot),median(PA_boot),stdev(PA_boot)
   print,'f1:    ',min(f1_boot),max(f1_boot),mean(f1_boot),median(f1_boot),stdev(f1_boot)
   print,'f2:    ',min(f2_boot),max(f2_boot),mean(f2_boot),median(f2_boot),stdev(f2_boot)
   print,'f3:    ',min(f3_boot),max(f3_boot),mean(f3_boot),median(f3_boot),stdev(f3_boot)
   print,'diam1: ',min(diam1_boot),max(diam1_boot),mean(diam1_boot),median(diam1_boot),stdev(diam1_boot)
   print,'diam2: ',min(diam2_boot),max(diam2_boot),mean(diam2_boot),median(diam2_boot),stdev(diam2_boot)
   print,'mu1:   ',min(mu1_boot),max(mu1_boot),mean(mu1_boot),median(mu1_boot),stdev(mu1_boot)
   print,'mu2:   ',min(mu2_boot),max(mu2_boot),mean(mu2_boot),median(mu2_boot),stdev(mu2_boot)
   print,'Number of bootstrap iterations:',nsamp
   
   ; compute 67% confidence ellipse
   amat = transpose([[ra_boot],[dec_boot]])
   covmat = correlate(amat,/covariance)
   eval = EIGENQL(covmat)
   chi2_67 = 2.25  ; chi2 for 67.5%, 2 variables
   boot_maj = sqrt(eval(0))*sqrt(2.25)
   boot_min = sqrt(eval(1))*sqrt(2.25)
   boot_angle = atan(eval(0)-covmat(0,0),covmat(0,1))*180.0/!dpi
   print,'67% confidence ellipse (maj,min,angle): ', boot_maj,boot_min,boot_angle

   create_ellipse,xe,ye,100,major=boot_maj,minor=boot_min,xcent=mean(ra_boot),ycent=mean(dec_boot),theta=boot_angle


   set_plot, 'ps'
   device, /Color, filename='temp_boot.eps',xsize=15,ysize=15,/encap

   plot,ra_boot,dec_boot,psym=7,/isotropic, $
     xrange=[max(ra_boot),min(ra_boot)],yrange=[min(dec_boot),max(dec_boot)], $
     xtitle='sepRA (mas)',ytitle='sepDEC (mas)'

   oplot,xe,ye,color=250
   
   device,/close
   set_plot,'x'
   
  print,'---------------------------------------------------------------'

endif
  
  print,'  sep      PA       sig_maj sig_min sig_PA   f1      sig_f1   f2      sig_f2   f3     sig_f3    diam1  sigd1   diam2  sigd2  chi2nu chi2nu '
  print,'  (mas)    (deg)    (mas)   (mas)   (deg)                                                       (mas)  (mas)   (mas)  (mas)  (V2)   (CP)'

  print,format='(f9.4,f9.3,f8.4,f8.4,f9.3,f9.5,f8.5,f9.5,f8.5,f9.5,f8.5,f8.4,f7.4,f8.4,f7.4,f7.2,f7.2)',sep,PA,sig_major_sys,sig_minor_sys,sig_pa_sys,params_best(2),sig_f1_sys,params_best(3),sig_f2_sys,params_best(4),sig_f3_sys,params_best(5),sig_diam1_sys,params_best(6),sig_diam2_sys,redchi_v2,redchi_t3


  ;print,'Unscaled Residuals V2 - mean median stdev: ',mean(abs(resid_V2)),median(abs(resid_V2)),stdev(abs(resid_V2))
  ;print,'Unscaled Residuals T3 - mean median stdev: ',mean(abs(resid_T3)),median(abs(resid_T3)),stdev(abs(resid_T3))

  ;stop,'Type .cont to continue'

  ;;;;;;;;;;;;;;;;;;;;;;
  ; Summary of Results ;
  ; Save to file.      ;
  ;;;;;;;;;;;;;;;;;;;;;;

  openw,1,'temp_results'
 
  printf,1,'---------------------------------------------------------------'
  if (ans_cp) then begin
     printf,1,'V2 Data file: ', datafile
     printf,1,'T3 Data file: ', datafile_cp
  endif else begin
     printf,1,'Data file: ', datafile
  endelse
  printf,1,'Best fit total(resid^2) from mpfit: ',total(resid^2)
  printf,1,'All chi2:',chi2_all,num_all,redchi_all
  printf,1,'V2 chi2:',chi2_v2,num_v2,redchi_v2
  printf,1,'T3 chi2:',chi2_t3,num_t3,redchi_t3
  printf,1,'Number of free parameters:',nfreepar

  printf,1,'Parameter Error'
  for i=0, nparms-1 do printf,1,parlabel(i),params_best(i),perror_best(i)*sqrt(redchi_all)

  printf,1,'Close pair separation:  ',sep,' +/- ',dsep
  printf,1,'Close pair position angle: ',PA,' +/- ',dPA
  printf,1,'Close pair fratio (f2/f1): ',fratio,' +/- ',dfratio

  printf,1,'Flux ratio of wide component relative to primary: ',params_best(4)/params_best(2)

  printf,1,'Error ellipse - major, minor, PA:',sqrt(lam1),sqrt(lam2),theta

  if (ans_sys) then begin
  printf,1,'---------------------------------------------------------------'
  printf,1,'Add systematic calibration uncertainties in quadrature with formal errors:'
  printf,1,'- ',sys_label1
  printf,1,'- ',sys_label2
  printf,1,'Total error ellipse - major, minor, PA: ',sig_major_sys,sig_minor_sys,sig_pa_sys
  printf,1,'Total errors in f1, f2, f3: ',sig_f1_sys,sig_f2_sys,sig_f3_sys
  printf,1,'Total errors in diam1, diam2: ',sig_diam1_sys, sig_diam2_sys
  printf,1,'---------------------------------------------------------------'
  endif

  printf,1,'Wavelength correction factor: ',string(format='(f8.5)',wavecor),'    -- ',wavelabel

  if (do_wave) then printf,1,'Fit includes bandwidth smearing.' $
               else printf,1,'Fit assumes monochromatic light.'

  if (fit_v2) then printf,1,'Fitting only V2 data.' $
  else if (fit_t3) then printf,1,'Fitting only T3 data.' $
  else printf,1,'Fitting V2 and T3 data.'

  if (note_select ne '') then print,note_select

  close,1

  ;;;;;;;;;;;;;;;;;
  ; Summary plots ;
  ;;;;;;;;;;;;;;;;;

  loadct,0

  !p.multi=[0,2,2]
  plot,[vis2data.baseline],[vis2data.vis2data],psym=6,$
       xtit="Baseline (m)",ytit="Vis2"
  oplot,[vis2model.baseline],[vis2model.vis2data],psym=7

  plot,[vis2model.vis2data],[vis2data.vis2data],psym=6,$
            xtitle="Model Vis2",ytitle="Data Vis2"
  for i=0, num_v2-1 do oplot,[vis2model(i).vis2data,vis2model(i).vis2data],$
      [vis2data(i).vis2data-vis2data(i).vis2err,vis2data(i).vis2data+vis2data(i).vis2err]
  oplot,[0,2],[0,2]

  plot,[t3model.t3phi],[t3data.t3phi],psym=6,$
            xtitle="Model CP (degs)",ytitle="Data CP (degs)"
  for i=0, num_t3-1 do oplot,[t3model(i).t3phi,t3model(i).t3phi],$
      [t3data(i).t3phi-t3data(i).t3phierr,t3data(i).t3phi+t3data(i).t3phierr]
  oplot,[-180,180],[-180,180]

  save,ra_array,filename='ra_array.idl'
  save,dec_array,filename='dec_array.idl'
  save,f1_array,filename='f1_array.idl'
  save,f2_array,filename='f2_array.idl'
  save,f3_array,filename='f3_array.idl'
  save,diam1_array,filename='diam1_array.idl'
  save,diam2_array,filename='diam2_array.idl'
  save,mu1_array,filename='mu1_array.idl'
  save,mu2_array,filename='mu2_array.idl'
  save,chi2_array,filename='chi2_array.idl'

  delvar,datafile_cp

  !p.multi=0

;stop
  
end
